@@ -38,7 +38,6 @@ exception statement from your version. */
 
 package gnu.java.awt.peer.gtk;
 
-import java.awt.AWTEvent;
 import java.awt.Color;
 import java.awt.Component;
 import java.awt.Container;
@@ -46,7 +45,6 @@ import java.awt.Font;
 import java.awt.Graphics;
 import java.awt.Insets;
 import java.awt.Window;
-import java.awt.event.PaintEvent;
 import java.awt.peer.ComponentPeer;
 import java.awt.peer.ContainerPeer;
 
@@ -117,9 +115,14 @@ public class GtkContainerPeer extends GtkComponentPeer
     Component[] components = ((Container) awtComponent).getComponents();
     for (int i = 0; i < components.length; i++)
       {
-        GtkComponentPeer peer = (GtkComponentPeer) components[i].getPeer();
-        if (peer != null && ! peer.awtComponent.isFontSet())
-          peer.setFont(f);
+        if (components[i].isLightweight ())
+          components[i].setFont (f);
+        else
+          {
+            GtkComponentPeer peer = (GtkComponentPeer) components[i].getPeer();
+            if (peer != null && ! peer.awtComponent.isFontSet())
+              peer.setFont(f);
+          }
       }
   }
 
