@@ -38,11 +38,8 @@ exception statement from your version. */
 
 package gnu.java.awt.peer.gtk;
 
-import java.awt.Component;
-import java.awt.Container;
 import java.awt.Frame;
 import java.awt.Graphics;
-import java.awt.Graphics2D;
 import java.awt.Image;
 import java.awt.MenuBar;
 import java.awt.Rectangle;
@@ -169,36 +166,17 @@ public class GtkFramePeer extends GtkWindowPeer
     setIconImage(frame.getIconImage());
   }
 
-  native void nativeSetIconImageFromDecoder (GdkPixbufDecoder decoder);
-  native void nativeSetIconImageFromData (int[] pixels, int width, int height);
+  native void nativeSetIconImage (GtkImage image);
+
   public void setIconImage (Image image) 
   {
-      if (image != null && image instanceof GtkImage)
-        {
-          GtkImage img = (GtkImage) image;
-          // FIXME: Image should be loaded, but if not, do image loading here.
-          if (img.isLoaded())
-            {
-              if (img.getSource() instanceof GdkPixbufDecoder)
-                {
-                  nativeSetIconImageFromDecoder((GdkPixbufDecoder) img.getSource());
-                }
-              else
-                {
-                  int[] pixels = img.getPixelCache();
-                  ColorModel model = img.getColorModel();
-                  int[] data = new int[pixels.length * 4];
-                  for (int i = 0; i < pixels.length; i++)
-                    {
-                      data[i * 4] = model.getRed(pixels[i]);
-                      data[i * 4 + 1] = model.getGreen(pixels[i]);
-                      data[i * 4 + 2] = model.getBlue(pixels[i]);
-                      data[i * 4 + 3] = model.getAlpha(pixels[i]);
-                    }
-                  nativeSetIconImageFromData(data, img.getWidth(null), img.getHeight(null));
-                }
-            }
-        }
+      if (image != null)
+	{
+	  if (image instanceof GtkImage)
+	    nativeSetIconImage((GtkImage) image);
+	  else
+	    nativeSetIconImage(new GtkImage(image.getSource()));
+	}
   }
 
   public Graphics getGraphics ()
@@ -253,7 +231,7 @@ public class GtkFramePeer extends GtkWindowPeer
   protected void postExposeEvent (int x, int y, int width, int height)
   {
     if (!isInRepaint)
-      q.postEvent (new PaintEvent (awtComponent, PaintEvent.PAINT,
+      q().postEvent (new PaintEvent (awtComponent, PaintEvent.PAINT,
                                    new Rectangle (x + insets.left, 
                                                   y + insets.top, 
                                                   width, height)));
