@@ -38,15 +38,14 @@ exception statement from your version. */
 
 package gnu.java.net.protocol.http;
 
-import gnu.java.net.protocol.http.event.RequestEvent;
 import gnu.java.net.BASE64;
 import gnu.java.net.LineInputStream;
+import gnu.java.net.protocol.http.event.RequestEvent;
 
-import java.io.InputStream;
 import java.io.IOException;
+import java.io.InputStream;
 import java.io.OutputStream;
 import java.net.ProtocolException;
-import java.net.Socket;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.text.DateFormat;
@@ -449,6 +448,7 @@ public class Request
       {
       case 204:
       case 205:
+      case 304:
         break;
       default:
         // Does response body reader want body?
