@@ -39,23 +39,19 @@ exception statement from your version. */
 
 package gnu.java.rmi.server;
 
-import gnu.java.rmi.server.RMIIncomingThread;
-import gnu.java.rmi.server.UnicastConnection;
-
 import java.io.IOException;
-import java.io.ObjectOutput;
 import java.io.ObjectInput;
+import java.io.ObjectOutput;
 import java.net.InetAddress;
-import java.net.Socket;
 import java.net.ServerSocket;
+import java.net.Socket;
 import java.net.UnknownHostException;
 import java.rmi.RemoteException;
-import java.rmi.server.RMISocketFactory;
-import java.rmi.server.RMIServerSocketFactory;
 import java.rmi.server.RMIClientSocketFactory;
+import java.rmi.server.RMIServerSocketFactory;
+import java.rmi.server.RMISocketFactory;
 import java.util.ArrayList;
 import java.util.ConcurrentModificationException;
-import java.util.Enumeration;
 import java.util.Hashtable;
 import java.util.Iterator;
 
