@@ -1,5 +1,5 @@
 /* BorderLayout.java -- A layout manager class
-   Copyright (C) 1999, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2002, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -43,7 +43,7 @@ package java.awt;
   * in certain sectors of the parent container.
   *
   * @author Aaron M. Renn (arenn@urbanophile.com)
-  * @author Rolf W. Rasmussen  <rolfwr@ii.uib.no>
+  * @author Rolf W. Rasmussen  (rolfwr@ii.uib.no)
   */
 public class BorderLayout implements LayoutManager2, java.io.Serializable
 {
@@ -700,6 +700,10 @@ calcSize(Container target, int what)
       Dimension cdim = calcCompSize(center, what);
 
       int width = edim.width + cdim.width + wdim.width + (hgap * 2);
+      // check for overflow
+      if (width < edim.width || width < cdim.width || width < cdim.width)
+          width = Integer.MAX_VALUE;
+
       if (ndim.width > width)
 	width = ndim.width;
       if (sdim.width > width)
@@ -713,7 +717,13 @@ calcSize(Container target, int what)
       if (wdim.height > height)
 	height = wdim.height;
 
-      height += (ndim.height + sdim.height + (vgap * 2) + ins.top + ins.bottom);
+      int addedHeight = height + (ndim.height + sdim.height + (vgap * 2)
+                                  + ins.top + ins.bottom);
+      // check for overflow
+      if (addedHeight < height)
+          height = Integer.MAX_VALUE;
+      else
+          height = addedHeight;
 
       return(new Dimension(width, height));
     }
