@@ -1,5 +1,5 @@
 /* DefaultKeyboardFocusManager.java -- 
-   Copyright (C) 2002 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -38,8 +38,15 @@ exception statement from your version. */
 
 package java.awt;
 
-import java.util.*;
-import java.awt.event.*;
+import java.awt.event.ActionEvent;
+import java.awt.event.FocusEvent;
+import java.awt.event.KeyEvent;
+import java.awt.event.WindowEvent;
+import java.util.Iterator;
+import java.util.LinkedList;
+import java.util.Set;
+import java.util.SortedSet;
+import java.util.TreeSet;
 
 // FIXME: finish documentation
 public class DefaultKeyboardFocusManager extends KeyboardFocusManager
@@ -133,6 +140,12 @@ public class DefaultKeyboardFocusManager extends KeyboardFocusManager
     }
   }
 
+  /**
+   * This flag indicates for which focus traversal key release event we
+   * possibly wait, before letting any more KEY_TYPED events through.
+   */
+  private AWTKeyStroke waitForKeyStroke = null;
+
   /** The {@link java.util.SortedSet} of current {@link
       #EventDelayRequest}s. */
   private SortedSet delayRequests = new TreeSet ();
@@ -164,18 +177,15 @@ public class DefaultKeyboardFocusManager extends KeyboardFocusManager
 
         if (e.id == FocusEvent.FOCUS_GAINED)
           {
-            if (((FocusEvent) e).isTemporary ())
-              setGlobalFocusOwner (target);
-            else
-              setGlobalPermanentFocusOwner (target);
-          }
-        else if (e.id == FocusEvent.FOCUS_LOST)
-          {
-            // We need to set the window's focus owner here; we can't
-            // set it when the window loses focus because by that time
-            // the previous focus owner has already lost focus
-            // (FOCUS_LOST events are delivered before
-            // WINDOW_LOST_FOCUS events).
+            if (! (target instanceof Window))
+              {
+                if (((FocusEvent) e).isTemporary ())
+                  setGlobalFocusOwner (target);
+                else
+                  setGlobalPermanentFocusOwner (target);
+              }
+
+            // Keep track of this window's focus owner.
 
             // Find the target Component's top-level ancestor.
             Container parent = target.getParent ();
@@ -188,9 +198,12 @@ public class DefaultKeyboardFocusManager extends KeyboardFocusManager
               (Window) target : (Window) parent;
 
             Component focusOwner = getFocusOwner ();
-            if (focusOwner != null)
+            if (focusOwner != null
+                && ! (focusOwner instanceof Window))
               toplevel.setFocusOwner (focusOwner);
-
+          }
+        else if (e.id == FocusEvent.FOCUS_LOST)
+          {
             if (((FocusEvent) e).isTemporary ())
               setGlobalFocusOwner (null);
             else
@@ -311,7 +324,7 @@ public class DefaultKeyboardFocusManager extends KeyboardFocusManager
                         MenuItem item = menu.getItem (j);
                         MenuShortcut shortcut = item.getShortcut ();
 
-                        if (shortcut != null)
+                        if (item.isEnabled() && shortcut != null)
                           {
                             // Dispatch a new ActionEvent if:
                             //
@@ -350,10 +363,33 @@ public class DefaultKeyboardFocusManager extends KeyboardFocusManager
     // the other two key event types for the same key (e.g. if
     // KEY_PRESSED TAB is a focus traversal keystroke, we also need to
     // consume KEY_RELEASED and KEY_TYPED TAB key events).
+    // consuming KEY_RELEASED is easy, because their keyCodes matches
+    // the KEY_PRESSED event. Consuming the intermediate KEY_TYPED is
+    // very difficult because their is no clean way that we can know
+    // which KEY_TYPED belongs to a focusTraversalKey and which not.
+    // To address this problem we swallow every KEY_TYPE between the
+    // KEY_PRESSED event that matches a focusTraversalKey and the
+    // corresponding KEY_RELEASED.
     AWTKeyStroke oppositeKeystroke = AWTKeyStroke.getAWTKeyStroke (e.getKeyCode (),
                                                                    e.getModifiersEx (),
                                                                    !(e.id == KeyEvent.KEY_RELEASED));
 
+    // Here we check if we are currently waiting for a KEY_RELEASED and
+    // swallow all KeyEvents that are to be delivered in between. This
+    // should only be the KEY_TYPED events that correspond to the
+    // focusTraversalKey's KEY_PRESSED event
+    if (waitForKeyStroke != null)
+      {
+        if (eventKeystroke.equals(waitForKeyStroke))
+          // release this lock
+          waitForKeyStroke = null;
+
+        // as long as we are waiting for the KEY_RELEASED, we swallow every
+        // KeyEvent, including the KEY_RELEASED
+        e.consume();
+        return;
+      }
+
     Set forwardKeystrokes = comp.getFocusTraversalKeys (KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS);
     Set backwardKeystrokes = comp.getFocusTraversalKeys (KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS);
     Set upKeystrokes = comp.getFocusTraversalKeys (KeyboardFocusManager.UP_CYCLE_TRAVERSAL_KEYS);
@@ -363,31 +399,29 @@ public class DefaultKeyboardFocusManager extends KeyboardFocusManager
 
     if (forwardKeystrokes.contains (eventKeystroke))
       {
+        waitForKeyStroke = oppositeKeystroke;        
         focusNextComponent (comp);
         e.consume ();
       }
     else if (backwardKeystrokes.contains (eventKeystroke))
       {
+        waitForKeyStroke = oppositeKeystroke;        
         focusPreviousComponent (comp);
         e.consume ();
       }
     else if (upKeystrokes.contains (eventKeystroke))
       {
+        waitForKeyStroke = oppositeKeystroke;        
         upFocusCycle (comp);
         e.consume ();
       }
     else if (comp instanceof Container
              && downKeystrokes.contains (eventKeystroke))
       {
+        waitForKeyStroke = oppositeKeystroke;        
         downFocusCycle ((Container) comp);
         e.consume ();
       }
-    else if (forwardKeystrokes.contains (oppositeKeystroke)
-             || backwardKeystrokes.contains (oppositeKeystroke)
-             || upKeystrokes.contains (oppositeKeystroke)
-             || (comp instanceof Container &&
-                 downKeystrokes.contains (oppositeKeystroke)))
-      e.consume ();
   }
 
   protected void enqueueKeyEvents (long after, Component untilFocused)
@@ -444,7 +478,7 @@ public class DefaultKeyboardFocusManager extends KeyboardFocusManager
 
     Component previous = policy.getComponentBefore (focusCycleRoot, focusComp);
     if (previous != null)
-    previous.requestFocusInWindow ();
+      previous.requestFocusInWindow ();
   }
 
   public void focusNextComponent (Component comp)
@@ -455,7 +489,7 @@ public class DefaultKeyboardFocusManager extends KeyboardFocusManager
 
     Component next = policy.getComponentAfter (focusCycleRoot, focusComp);
     if (next != null)
-    next.requestFocusInWindow ();
+      next.requestFocusInWindow ();
   }
 
   public void upFocusCycle (Component comp)
@@ -468,7 +502,7 @@ public class DefaultKeyboardFocusManager extends KeyboardFocusManager
         FocusTraversalPolicy policy = focusCycleRoot.getFocusTraversalPolicy ();
         Component defaultComponent = policy.getDefaultComponent (focusCycleRoot);
         if (defaultComponent != null)
-        defaultComponent.requestFocusInWindow ();
+          defaultComponent.requestFocusInWindow ();
       }
     else
       {
@@ -489,7 +523,7 @@ public class DefaultKeyboardFocusManager extends KeyboardFocusManager
         FocusTraversalPolicy policy = cont.getFocusTraversalPolicy ();
         Component defaultComponent = policy.getDefaultComponent (cont);
         if (defaultComponent != null)
-        defaultComponent.requestFocusInWindow ();
+          defaultComponent.requestFocusInWindow ();        
         setGlobalCurrentFocusCycleRoot (cont);
       }
   }
