@@ -1,39 +1,40 @@
 /* GeneralPath.java -- represents a shape built from subpaths
    Copyright (C) 2002, 2003, 2004 Free Software Foundation
 
-   This file is part of GNU Classpath.
+This file is part of GNU Classpath.
 
-   GNU Classpath is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+GNU Classpath is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2, or (at your option)
+any later version.
 
-   GNU Classpath is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+GNU Classpath is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Classpath; see the file COPYING.  If not, write to the
-   Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-   02111-1307 USA.
+You should have received a copy of the GNU General Public License
+along with GNU Classpath; see the file COPYING.  If not, write to the
+Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+02111-1307 USA.
 
-   Linking this library statically or dynamically with other modules is
-   making a combined work based on this library.  Thus, the terms and
-   conditions of the GNU General Public License cover the whole
-   combination.
+Linking this library statically or dynamically with other modules is
+making a combined work based on this library.  Thus, the terms and
+conditions of the GNU General Public License cover the whole
+combination.
+
+As a special exception, the copyright holders of this library give you
+permission to link this library with independent modules to produce an
+executable, regardless of the license terms of these independent
+modules, and to copy and distribute the resulting executable under
+terms of your choice, provided that you also meet, for each linked
+independent module, the terms and conditions of the license of that
+module.  An independent module is a module which is not derived from
+or based on this library.  If you modify this library, you may extend
+this exception to your version of the library, but you are not
+obligated to do so.  If you do not wish to do so, delete this
+exception statement from your version. */
 
-   As a special exception, the copyright holders of this library give you
-   permission to link this library with independent modules to produce an
-   executable, regardless of the license terms of these independent
-   modules, and to copy and distribute the resulting executable under
-   terms of your choice, provided that you also meet, for each linked
-   independent module, the terms and conditions of the license of that
-   module.  An independent module is a module which is not derived from
-   or based on this library.  If you modify this library, you may extend
-   this exception to your version of the library, but you are not
-   obligated to do so.  If you do not wish to do so, delete this
-   exception statement from your version. */
 
 package java.awt.geom;
 
@@ -87,15 +88,18 @@ public final class GeneralPath implements Shape, Cloneable
   /** A big number, but not so big it can't survive a few float operations */
   private static final double BIG_VALUE = java.lang.Double.MAX_VALUE / 10.0;
 
-  /** The winding rule. */
-  private int rule;
+  /** The winding rule.
+   * This is package-private to avoid an accessor method.
+   */
+  int rule;
 
   /**
    * The path type in points. Note that xpoints[index] and ypoints[index] maps
    * to types[index]; the control points of quad and cubic paths map as
    * well but are ignored.
+   * This is package-private to avoid an accessor method.
    */
-  private byte[] types;
+  byte[] types;
 
   /**
    * The list of all points seen. Since you can only append floats, it makes
@@ -103,15 +107,18 @@ public final class GeneralPath implements Shape, Cloneable
    * allow a general path of double precision points.
    * Note: Storing x and y coords seperately makes for a slower transforms,
    * But it speeds up and simplifies box-intersection checking a lot.
+   * These are package-private to avoid accessor methods.
    */
-  private float[] xpoints;
-  private float[] ypoints;
+  float[] xpoints;
+  float[] ypoints;
 
   /** The index of the most recent moveto point, or null. */
   private int subpath = -1;
 
-  /** The next available index into points. */
-  private int index;
+  /** The next available index into points.
+   * This is package-private to avoid an accessor method.
+   */
+  int index;
 
   /**
    * Constructs a GeneralPath with the default (NON_ZERO)
@@ -533,8 +540,9 @@ public final class GeneralPath implements Shape, Cloneable
 
     /**
      * The GeneralPath whose segments are being iterated.
+     * This is package-private to avoid an accessor method.
      */
-    private final GeneralPath path;
+    final GeneralPath path;
 
     /**
      * The affine transformation used to transform coordinates.
@@ -641,7 +649,7 @@ public final class GeneralPath implements Shape, Cloneable
 	  if (transform != null)
 	    transform.transform( /* src */
 	    coords, /* srcOffset */
-	    pos, /* dest */ coords, /* destOffset */
+	    0, /* dest */ coords, /* destOffset */
 	    0, /* numPoints */ numCoords);
         }
       return seg;
@@ -777,7 +785,10 @@ public final class GeneralPath implements Shape, Cloneable
 
     /* Get a value which is hopefully small but not insignificant relative
      the path. */
-    epsilon = ypoints[0] * 1E-9;
+    epsilon = ypoints[0] * 1E-7;
+
+    if(epsilon == 0) 
+      epsilon = 1E-7;
 
     pos = 0;
     while (pos < index)
