@@ -448,11 +448,12 @@ public class BufferedImage extends Image
   {
     return new ImageProducer() {
         
-        HashSet consumers = new HashSet();
+	Vector consumers = new Vector();
 
         public void addConsumer(ImageConsumer ic)
         {
-          consumers.add(ic);
+	  if(!consumers.contains(ic))
+	    consumers.add(ic);
         }
 
         public boolean isConsumer(ImageConsumer ic)
@@ -462,7 +463,7 @@ public class BufferedImage extends Image
 
         public void removeConsumer(ImageConsumer ic)
         {
-          consumers.remove(ic);
+	  consumers.remove(ic);
         }
 
         public void startProduction(ImageConsumer ic)
@@ -480,10 +481,9 @@ public class BufferedImage extends Image
 
           consumers.add(ic);
 
-          Iterator i = consumers.iterator();
-          while(i.hasNext())
+	  for(int i=0;i<consumers.size();i++)
             {
-              ImageConsumer c = (ImageConsumer) i.next();
+              ImageConsumer c = (ImageConsumer) consumers.elementAt(i);
               c.setHints(ImageConsumer.SINGLEPASS);
               c.setDimensions(getWidth(), getHeight());
               c.setPixels(x, y, width, height, model, pixels, offset, stride);
