@@ -1,4 +1,4 @@
-/* Copyright (C) 2000, 2001, 2002  Free Software Foundation
+/* Copyright (C) 2000, 2001, 2002, 2005  Free Software Foundation
 
 This file is part of GNU Classpath.
 
@@ -37,7 +37,7 @@ exception statement from your version. */
 package java.awt.image;
 
 /**
- * @author Rolf W. Rasmussen <rolfwr@ii.uib.no>
+ * @author Rolf W. Rasmussen (rolfwr@ii.uib.no)
  */
 public abstract class SampleModel
 {
@@ -236,7 +236,7 @@ public abstract class SampleModel
       {
 	for (int xx=x; xx<(x+w); xx++)
 	  {
-	    getPixel(xx, yy, pixel, data);
+	    pixel = getPixel(xx, yy, pixel, data);
 	    System.arraycopy(pixel, 0, iArray, outOffset, numBands);
 	    outOffset += numBands;
 	  }
@@ -257,7 +257,7 @@ public abstract class SampleModel
       {
 	for (int xx=x; xx<(x+w); xx++)
 	  {
-	    getPixel(xx, yy, pixel, data);
+	    pixel = getPixel(xx, yy, pixel, data);
 	    System.arraycopy(pixel, 0, fArray, outOffset, numBands);
 	    outOffset += numBands;
 	  }
@@ -278,7 +278,7 @@ public abstract class SampleModel
       {
 	for (int xx=x; xx<(x+w); xx++)
 	  {
-	    getPixel(xx, yy, pixel, data);
+	    pixel = getPixel(xx, yy, pixel, data);
 	    System.arraycopy(pixel, 0, dArray, outOffset, numBands);
 	    outOffset += numBands;
 	  }
