@@ -97,7 +97,8 @@ final class DeleteFileHelper extends Thread
       }
   }
 
-  private DeleteFileHelper()
+  // Package-private to avoid a trampoline constructor.
+  DeleteFileHelper()
   {
   }
   
