@@ -58,6 +58,19 @@ package java.io;
  */
 public class PrintStream extends FilterOutputStream
 {
+  /* Notice the implementation is quite similar to OutputStreamWriter.
+   * This leads to some minor duplication, because neither inherits
+   * from the other, and we want to maximize performance. */
+
+  // Line separator string.
+  private static final char[] line_separator
+    = System.getProperty("line.separator").toCharArray();
+
+  /**
+   *  Encoding name
+   */
+  private String encoding;
+
   /**
    * This boolean indicates whether or not an error has ever occurred
    * on this stream.
@@ -70,54 +83,6 @@ public class PrintStream extends FilterOutputStream
    */
   private boolean auto_flush;
 
-  /**
-   * The PrintWriter instance this object writes to
-   */
-  private PrintWriter pw;
-
-  /**
-   * Lets us know if the stream is closed
-   */
-  private boolean closed;
-
-  /**
-   * This class exists to forward the write calls from the PrintWriter back
-   * to us. This is required to make subclassing of PrintStream work
-   * correctly.
-   */
-  private class ForwardStream extends OutputStream
-  {
-    // This is package-private to avoid a trampoline constructor.
-    ForwardStream ()
-    {
-    }
-
-    public void close () throws IOException
-    {
-      out.close ();
-    }
-
-    public void flush () throws IOException
-    {
-      out.flush ();
-    }
-
-    public void write (byte[] b) throws IOException
-    {
-	PrintStream.this.write (b);
-    }
-
-    public void write (byte[] b, int off, int len) throws IOException
-    {
-	PrintStream.this.write (b, off, len);
-    }
-
-    public void write (int b) throws IOException
-    {
-	PrintStream.this.write (b);
-    }
-  }
-
   /**
    * This method intializes a new <code>PrintStream</code> object to write
    * to the specified output sink.
@@ -146,10 +111,15 @@ public class PrintStream extends FilterOutputStream
   {
     super (out);
 
-    // FIXME Instead of using PrintWriter and ForwardStream we
-    // should inline the character conversion (see libgcj's version
-    // of this class)
-    pw = new PrintWriter (new ForwardStream (), auto_flush);
+    try {
+	this.encoding = System.getProperty("file.encoding");
+    } catch (SecurityException e){
+	this.encoding = "ISO8859_1";
+    } catch (IllegalArgumentException e){
+	this.encoding = "ISO8859_1";
+    } catch (NullPointerException e){
+	this.encoding = "ISO8859_1";
+    }
     this.auto_flush = auto_flush;
   }
 
@@ -173,12 +143,8 @@ public class PrintStream extends FilterOutputStream
   {
     super (out);
 
-    // FIXME Instead of using PrintWriter and ForwardStream we
-    // should inline the character conversion (see libgcj's version
-    // of this class)
-    pw = new PrintWriter (
-	    new OutputStreamWriter (
-		new ForwardStream (), encoding), auto_flush);
+    new String(new byte[]{0}, encoding);    // check if encoding is supported
+    this.encoding = encoding;
     this.auto_flush = auto_flush;
   }
 
@@ -193,10 +159,8 @@ public class PrintStream extends FilterOutputStream
    */
   public boolean checkError ()
   {
-    if (!closed)
-      flush ();
-
-    return error_occurred | pw.checkError ();
+    flush ();
+    return error_occurred;
   }
 
   /**
@@ -213,8 +177,19 @@ public class PrintStream extends FilterOutputStream
    */
   public void close ()
   {
-    pw.close ();
-    closed = true;
+    try
+      {
+	flush();
+	out.close();
+      }
+    catch (InterruptedIOException iioe)
+      {
+	Thread.currentThread().interrupt();
+      }
+    catch (IOException e)
+      {
+	setError ();
+      }
   }
 
   /**
@@ -223,7 +198,73 @@ public class PrintStream extends FilterOutputStream
    */
   public void flush ()
   {
-    pw.flush();
+    try
+      {
+	out.flush();
+      }
+    catch (InterruptedIOException iioe)
+      {
+	Thread.currentThread().interrupt();
+      }
+    catch (IOException e)
+      {
+	setError ();
+      }
+  }
+
+  private synchronized void print (String str, boolean println)
+  {
+    try
+      {
+        writeChars(str, 0, str.length());
+	if (println)
+	  writeChars(line_separator, 0, line_separator.length);
+	if (auto_flush)
+	  flush();
+      }
+    catch (InterruptedIOException iioe)
+      {
+	Thread.currentThread().interrupt();
+      }
+    catch (IOException e)
+      {
+	setError ();
+      }
+  }
+
+  private synchronized void print (char[] chars, int pos, int len,
+				   boolean println)
+  {
+    try
+      {
+        writeChars(chars, pos, len);
+	if (println)
+	  writeChars(line_separator, 0, line_separator.length);
+	if (auto_flush)
+	  flush();
+      }
+    catch (InterruptedIOException iioe)
+      {
+	Thread.currentThread().interrupt();
+      }
+    catch (IOException e)
+      {
+	setError ();
+      }
+  }
+
+  private void writeChars(char[] buf, int offset, int count)
+    throws IOException
+  {
+      byte[] bytes = (new String(buf, offset, count)).getBytes(encoding);
+      out.write(bytes, 0, bytes.length);
+  }
+
+  private void writeChars(String str, int offset, int count)
+    throws IOException
+  {
+      byte[] bytes = str.substring(offset, offset+count).getBytes(encoding);
+      out.write(bytes, 0, bytes.length);
   }
 
   /**
@@ -235,7 +276,7 @@ public class PrintStream extends FilterOutputStream
    */
   public void print (boolean bool)
   {
-    print (String.valueOf (bool));
+    print(String.valueOf(bool), false);
   }
 
   /**
@@ -246,7 +287,7 @@ public class PrintStream extends FilterOutputStream
    */
   public void print (int inum)
   {
-    print (String.valueOf (inum));
+    print(String.valueOf(inum), false);
   }
 
   /**
@@ -257,7 +298,7 @@ public class PrintStream extends FilterOutputStream
    */
   public void print (long lnum)
   {
-    print (String.valueOf (lnum));
+    print(String.valueOf(lnum), false);
   }
 
   /**
@@ -268,7 +309,7 @@ public class PrintStream extends FilterOutputStream
    */
   public void print (float fnum)
   {
-    print (String.valueOf (fnum));
+    print(String.valueOf(fnum), false);
   }
 
   /**
@@ -279,7 +320,7 @@ public class PrintStream extends FilterOutputStream
    */
   public void print (double dnum)
   {
-    print (String.valueOf (dnum));
+    print(String.valueOf(dnum), false);
   }
 
   /**
@@ -291,9 +332,7 @@ public class PrintStream extends FilterOutputStream
    */
   public void print (Object obj)
   {
-    // Don't call pw directly.  Convert to String so we scan for newline
-    // characters on auto-flush;
-    print (String.valueOf (obj));
+    print(obj == null ? "null" : obj.toString(), false);
   }
 
   /**
@@ -304,10 +343,7 @@ public class PrintStream extends FilterOutputStream
    */
   public void print (String str)
   {
-    pw.print (str);
-
-    if (auto_flush)
-      flush ();
+    print(str == null ? "null" : str, false);
   }
 
   /**
@@ -316,9 +352,9 @@ public class PrintStream extends FilterOutputStream
    *
    * @param ch The <code>char</code> value to be printed
    */
-  public void print (char ch)
+  public synchronized void print (char ch)
   {
-    print (String.valueOf (ch));
+    print(new char[]{ch}, 0, 1, false);
   }
 
   /**
@@ -329,7 +365,7 @@ public class PrintStream extends FilterOutputStream
    */
   public void print (char[] charArray)
   {
-    pw.print (charArray);
+    print(charArray, 0, charArray.length, false);
   }
 
   /**
@@ -339,7 +375,7 @@ public class PrintStream extends FilterOutputStream
    */
   public void println ()
   {
-    pw.println();
+    print(line_separator, 0, line_separator.length, false);
   }
 
   /**
@@ -353,7 +389,7 @@ public class PrintStream extends FilterOutputStream
    */
   public void println (boolean bool)
   {
-    println (String.valueOf (bool));
+    print(String.valueOf(bool), true);
   }
 
   /**
@@ -366,7 +402,7 @@ public class PrintStream extends FilterOutputStream
    */
   public void println (int inum)
   {
-    println (String.valueOf (inum));
+    print(String.valueOf(inum), true);
   }
 
   /**
@@ -379,7 +415,7 @@ public class PrintStream extends FilterOutputStream
    */
   public void println (long lnum)
   {
-    println (String.valueOf (lnum));
+    print(String.valueOf(lnum), true);
   }
 
   /**
@@ -392,7 +428,7 @@ public class PrintStream extends FilterOutputStream
    */
   public void println (float fnum)
   {
-    println (String.valueOf (fnum));
+    print(String.valueOf(fnum), true);
   }
 
   /**
@@ -405,7 +441,7 @@ public class PrintStream extends FilterOutputStream
    */
   public void println (double dnum)
   {
-    println (String.valueOf (dnum));
+    print(String.valueOf(dnum), true);
   }
 
   /**
@@ -419,7 +455,7 @@ public class PrintStream extends FilterOutputStream
    */
   public void println (Object obj)
   {
-    println (String.valueOf (obj));
+    print(obj == null ? "null" : obj.toString(), true);
   }
 
   /**
@@ -432,7 +468,7 @@ public class PrintStream extends FilterOutputStream
    */
   public void println (String str)
   {
-    pw.println (str);
+    print (str == null ? "null" : str, true);
   }
 
   /**
@@ -443,9 +479,9 @@ public class PrintStream extends FilterOutputStream
    *
    * @param ch The <code>char</code> value to be printed
    */
-  public void println (char ch)
+  public synchronized void println (char ch)
   {
-    println (String.valueOf (ch));
+    print(new char[]{ch}, 0, 1, true);
   }
 
   /**
@@ -458,7 +494,7 @@ public class PrintStream extends FilterOutputStream
    */
   public void println (char[] charArray)
   {
-    pw.println (charArray);
+    print(charArray, 0, charArray.length, true);
   }
 
   /**
@@ -470,10 +506,6 @@ public class PrintStream extends FilterOutputStream
    */
   public void write (int oneByte)
   {
-    // We actually have to implement this method. Flush first so that
-    // things get written in the right order.
-    flush();
-
     try
       {
         out.write (oneByte & 0xff);
@@ -501,10 +533,6 @@ public class PrintStream extends FilterOutputStream
    */
   public void write (byte[] buffer, int offset, int len)
   {
-    // We actually have to implement this method too. Flush first so that
-    // things get written in the right order.
-    flush();
-
     try
       {
         out.write (buffer, offset, len);
@@ -522,3 +550,4 @@ public class PrintStream extends FilterOutputStream
       }
   }
 } // class PrintStream
+
