@@ -1,5 +1,5 @@
 /* SecurityManager.java -- security checks for privileged actions
-   Copyright (C) 1998, 1999, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1998, 1999, 2001, 2002, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -38,6 +38,8 @@ exception statement from your version. */
 
 package java.lang;
 
+import gnu.classpath.VMStackWalker;
+
 import java.awt.AWTPermission;
 import java.io.File;
 import java.io.FileDescriptor;
@@ -105,7 +107,7 @@ import java.util.StringTokenizer;
  * subclassing this class.
  *
  * @author John Keiser
- * @author Eric Blake <ebb9@email.byu.edu>
+ * @author Eric Blake (ebb9@email.byu.edu)
  * @see ClassLoader
  * @see SecurityException
  * @see #checkTopLevelWindow(Object)
@@ -179,7 +181,10 @@ public class SecurityManager
    */
   protected Class[] getClassContext()
   {
-    return VMSecurityManager.getClassContext();
+    Class[] stack1 = VMStackWalker.getClassContext();
+    Class[] stack2 = new Class[stack1.length - 1];
+    System.arraycopy(stack1, 1, stack2, 0, stack1.length - 1);
+    return stack2;
   }
 
   /**
@@ -201,7 +206,8 @@ public class SecurityManager
    */
   protected ClassLoader currentClassLoader()
   {
-    return VMSecurityManager.currentClassLoader();
+    Class cl = currentLoadedClass();
+    return cl != null ? cl.getClassLoader() : null;
   }
 
   /**
@@ -1027,13 +1033,15 @@ public class SecurityManager
                         String permission)
   {
     if (packageName == null)
-	throw new NullPointerException();
+      throw new NullPointerException();
 
-    String list = (String)AccessController.doPrivileged(new PrivilegedAction() {
-	public Object run() {
-	    return Security.getProperty(restriction);
+    String list = (String)AccessController.doPrivileged(new PrivilegedAction()
+      {
+	public Object run()
+        {
+	  return Security.getProperty(restriction);
 	}
-    });
+      });
 
     if (list == null || list.equals(""))
       return;
@@ -1051,12 +1059,4 @@ public class SecurityManager
 	  }
       }
   }
-} // class SecurityManager
-
-// XXX This class is unnecessary.
-class SecurityContext {
-	Class[] classes;
-	SecurityContext(Class[] classes) {
-		this.classes = classes;
-	}
 }
