@@ -180,7 +180,7 @@ public final class Inet4Address extends InetAddress
    */
   public byte[] getAddress()
   {
-    return addr;
+    return (byte[]) addr.clone();
   }
 
   /**
