@@ -39,69 +39,54 @@ package java.net;
 
 
 /**
-  * This class serves a container for username/password pairs.
-  *
-  * @author Aaron M. Renn (arenn@urbanophile.com)
-  * @since 1.2
-  */
+ * This class serves a container for username/password pairs.
+ *
+ * @author Aaron M. Renn (arenn@urbanophile.com)
+ *
+ * @since 1.2
+ */
 public final class PasswordAuthentication
 {
-  /*
-   * Instance Variables
-   */
-
   /**
-    * The username
-    */
+   * The username
+   */
   private String username;
 
   /**
-    * The password
-    */
+   * The password
+   */
   private char[] password;
 
-  /*************************************************************************/
-
-  /*
-   * Constructors
-   */
-
   /**
-    * Creates a new <code>PasswordAuthentication</code> object from the
-    * specified username and password.
-    *
-    * @param username The username for this object
-    * @param password The password for this object
-    */
+   * Creates a new <code>PasswordAuthentication</code> object from the
+   * specified username and password.
+   *
+   * @param username The username for this object
+   * @param password The password for this object
+   */
   public PasswordAuthentication(String username, char[] password)
   {
     this.username = username;
     this.password = password;
   }
 
-  /*************************************************************************/
-
-  /*
-   * Instance Methods
-   */
-
   /**
-    * Returns the username associated with this object
-    *
-    * @return The username
-    */
+   * Returns the username associated with this object
+   *
+   * @return The username
+   */
   public String getUserName()
   {
-    return (username);
+    return username;
   }
 
   /**
-    * Returns the password associated with this object
-    *
-    * @return The password
-    */
+   * Returns the password associated with this object
+   *
+   * @return The password
+   */
   public char[] getPassword()
   {
-    return (password);
+    return password;
   }
-} // class PasswordAuthentication
+}
