@@ -55,6 +55,11 @@ import java.util.StringTokenizer;
  */
 public class RMIClassLoader
 {
+  /**
+   * This class isn't intended to be instantiated.
+   */
+  private RMIClassLoader() {}
+
   private static class MyClassLoader extends URLClassLoader
   {
     // Package-private to avoid a trampoline constructor.
@@ -245,7 +250,7 @@ public class RMIClassLoader
    * 
    * @throws MalformedURLException if the codebase contains a malformed URL
    */
-  private static ClassLoader getClassLoader (String codebases) 
+  public static ClassLoader getClassLoader (String codebases) 
     throws MalformedURLException
   {
     ClassLoader loader;
