@@ -1,5 +1,5 @@
 /* Locale.java -- i18n locales
-   Copyright (C) 1998, 1999, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1998, 1999, 2001, 2002, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -39,7 +39,6 @@ exception statement from your version. */
 package java.util;
 
 import gnu.classpath.SystemProperties;
-import gnu.java.locale.LocaleHelper;
 
 import java.io.IOException;
 import java.io.ObjectInputStream;
@@ -454,33 +453,35 @@ public final class Locale implements Serializable, Cloneable
    */
   private static String[] getISOStrings(String tableName)
   {
-    List tempList;
-    ResourceBundle bundle;
-    Enumeration keys;
     int count = 0;
-    String[] strings;
+    ResourceBundle bundle =
+      ResourceBundle.getBundle("gnu.java.locale.LocaleInformation");
+    Enumeration e = bundle.getKeys();
+    ArrayList tempList = new ArrayList();
 
-    tempList = new ArrayList();
-    bundle = ResourceBundle.getBundle("gnu.java.locale.LocaleInformation");
-    keys = ((Hashtable) bundle.getObject(tableName)).keys();
-    while (keys.hasMoreElements())
+    while (e.hasMoreElements())
       {
-	String nextString;
-
-	nextString = (String) keys.nextElement();
-	if (nextString.length() == 2 &&
-	    Character.isLetter(nextString.charAt(0)) &&
-	    Character.isLetter(nextString.charAt(1)))
+	String key = (String) e.nextElement();
+	
+	if (key.startsWith(tableName + "."))
 	  {
-	    tempList.add(nextString);
-	    ++count;
+	    String str = key.substring(tableName.length() + 1);
+
+	    if (str.length() == 2
+		&& Character.isLetter(str.charAt(0))
+		&& Character.isLetter(str.charAt(1)))
+	      {
+		tempList.add(str);
+		++count;
+	      }
 	  }
       }
-    strings = new String[count];
+
+    String[] strings = new String[count];
+    
     for (int a = 0; a < count; ++a)
-      {
-	strings[a] = (String) tempList.get(a);
-      }
+      strings[a] = (String) tempList.get(a);
+    
     return strings;
   }
 
@@ -663,8 +664,19 @@ public final class Locale implements Serializable, Cloneable
    */
   public String getDisplayLanguage(Locale inLocale)
   {
-    return LocaleHelper.getLocalizedString(inLocale, language,
-					   "languages", true, false);
+    try
+      {
+	ResourceBundle res =
+          ResourceBundle.getBundle("gnu.java.locale.LocaleInformation",
+                                   inLocale,
+                                   ClassLoader.getSystemClassLoader());
+
+        return res.getString("languages." + language);
+      }
+    catch (MissingResourceException e)
+      {
+	return language;
+      }
   }
 
   /**
@@ -710,8 +722,19 @@ public final class Locale implements Serializable, Cloneable
    */
   public String getDisplayCountry(Locale inLocale)
   {
-    return LocaleHelper.getLocalizedString(inLocale, country,
-					    "territories", true, false);
+    try
+      {
+        ResourceBundle res =
+          ResourceBundle.getBundle("gnu.java.locale.LocaleInformation",
+                                   inLocale,
+                                   ClassLoader.getSystemClassLoader());
+    
+        return res.getString("territories." + country);
+      }
+    catch (MissingResourceException e)
+      {
+        return country;
+      }
   }
 
   /**
@@ -758,8 +781,19 @@ public final class Locale implements Serializable, Cloneable
    */
   public String getDisplayVariant(Locale inLocale)
   {
-    return LocaleHelper.getLocalizedString(inLocale, variant, "variants",
-					   true, false);
+    try
+      {
+        ResourceBundle res =
+          ResourceBundle.getBundle("gnu.java.locale.LocaleInformation",
+                                   inLocale,
+                                   ClassLoader.getSystemClassLoader());
+    
+        return res.getString("variants." + variant);
+      }
+    catch (MissingResourceException e)
+      {
+        return variant;
+      }
   }
 
   /**
