@@ -1,7 +1,5 @@
-/* Logger.java
-   -- a class for logging messages
-
-Copyright (C) 2002, 2004 Free Software Foundation, Inc.
+/* Logger.java -- a class for logging messages
+   Copyright (C) 2002, 2004 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -35,9 +33,7 @@ module.  An independent module is a module which is not derived from
 or based on this library.  If you modify this library, you may extend
 this exception to your version of the library, but you are not
 obligated to do so.  If you do not wish to do so, delete this
-exception statement from your version.
-
-*/
+exception statement from your version. */
 
 
 package java.util.logging;
@@ -1150,13 +1146,8 @@ public class Logger
     lm = LogManager.getLogManager();
 
     if (this == lm.rootLogger)
-    {
-      if (parent != null)
         throw new IllegalArgumentException(
           "only the root logger can have a null parent");
-      this.parent = null;
-      return;
-    }
 
     /* An application is allowed to control an anonymous logger
      * without having the permission to control the logging
