@@ -550,6 +550,8 @@ public abstract class AbstractPreferences extends Preferences {
             String value;
             try {
                 value = getSpi(key);
+            } catch (ThreadDeath death) {
+                throw death;
             } catch (Throwable t) {
                 value = null;
             }
@@ -1040,7 +1042,7 @@ public abstract class AbstractPreferences extends Preferences {
             } else {
                 flushSpi();
             }
-            keys = (String[]) childCache.keySet().toArray();
+            keys = (String[]) childCache.keySet().toArray(new String[]{});
         }
 
         if (keys != null) {
