@@ -1,5 +1,5 @@
 /* Pattern.java -- Compiled regular expression ready to be applied.
-   Copyright (C) 2002, 2004 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004, 2005 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -84,8 +84,7 @@ public final class Pattern implements Serializable
     // if ((flags & UNICODE_CASE) != 0) gnuFlags =
     // if ((flags & CANON_EQ) != 0) gnuFlags =
 
-    // Eventually there will be such a thing as JDK 1_4 syntax
-    RESyntax syntax = RESyntax.RE_SYNTAX_PERL5;
+    RESyntax syntax = RESyntax.RE_SYNTAX_JAVA_1_4;
     if ((flags & UNIX_LINES) != 0)
       {
 	// Use a syntax set with \n for linefeeds?
@@ -188,9 +187,9 @@ public final class Pattern implements Serializable
     int count = 0;
     int start = 0;
     int end;
-    boolean matched;
+    boolean matched = matcher.find();
 
-    while (matched = matcher.find() && (limit <= 0 || count < limit - 1))
+    while (matched && (limit <= 0 || count < limit - 1))
       {
 	++count;
 	end = matcher.start();
@@ -198,13 +197,17 @@ public final class Pattern implements Serializable
 	  empties++;
 	else
 	  {
-	    while (empties-- > 0)
-	      list.add("");
+	    while (empties > 0)
+	      {
+		list.add("");
+		empties--;
+	      }
 
 	    String text = input.subSequence(start, end).toString();
 	    list.add(text);
 	  }
 	start = matcher.end();
+	matched = matcher.find();
       }
 
     // We matched nothing.
@@ -222,8 +225,11 @@ public final class Pattern implements Serializable
 	    int max = limit - list.size();
 	    empties = (empties > max) ? max : empties;
 	  }
-	while (empties-- > 0)
-	  list.add("");
+	while (empties > 0)
+	  {
+	    list.add("");
+	    empties--;
+	  }
       }
 
     // last token at end
