@@ -38,13 +38,14 @@ exception statement from your version. */
 
 package javax.swing;
 
+import java.awt.AWTError;
+import java.awt.Component;
+import java.awt.Dimension;
+import java.awt.LayoutManager;
+
 import javax.accessibility.Accessible;
 import javax.accessibility.AccessibleContext;
 import javax.accessibility.AccessibleRole;
-import java.awt.LayoutManager;
-import java.awt.Component;
-import java.awt.Dimension;
-import java.awt.AWTError;
 
 /**
  * A component that uses a {@link BoxLayout} as Layout Manager.
@@ -191,7 +192,7 @@ public class Box extends JComponent implements Accessible
   public static Component createGlue()
   {
     Filler glue = new Filler(new Dimension(0,0), new Dimension(0,0),
-                             new Dimension(Integer.MAX_VALUE,Integer.MAX_VALUE)
+                             new Dimension(Short.MAX_VALUE,Short.MAX_VALUE)
                              );
     return glue;
   }
@@ -210,7 +211,10 @@ public class Box extends JComponent implements Accessible
    */
   public static Component createHorizontalGlue()
   {
-    return createGlue();
+    Filler glue = new Filler(new Dimension(0,0), new Dimension(0,0),
+                             new Dimension(Short.MAX_VALUE, 0)
+                             );
+    return glue;
   }
   
   /**
