@@ -51,7 +51,7 @@ import javax.swing.event.ChangeListener;
 import javax.swing.event.EventListenerList;
 
 /**
- * The purpose of this class is to model the dynamic state of an abstract
+ * The pUrpose of this class is to model the dynamic state of an abstract
  * button. The concrete button type holding this state may be a a "toggle"
  * button (checkbox, radio button) or a "push" button (menu button, button).
  * If the model is disabled, only the "selected" property can be changed. An
@@ -116,7 +116,6 @@ public class DefaultButtonModel implements ButtonModel, Serializable
    * this model.
    */
   protected EventListenerList listenerList = new EventListenerList();
-  ;
 
   /** The single ChangeEvent this model (re)uses to call its ChangeListeners. */
   protected ChangeEvent changeEvent = new ChangeEvent(this);
@@ -272,7 +271,7 @@ public class DefaultButtonModel implements ButtonModel, Serializable
    *
    * @param e The ItemEvent to fire
    */
-  public void fireItemStateChanged(ItemEvent e)
+  protected void fireItemStateChanged(ItemEvent e)
   {
     ItemListener[] ll = getItemListeners();
 
@@ -288,7 +287,7 @@ public class DefaultButtonModel implements ButtonModel, Serializable
    *
    * @param e The ActionEvent to fire
    */
-  public void fireActionPerformed(ActionEvent e)
+  protected void fireActionPerformed(ActionEvent e)
   {
     ActionListener[] ll = getActionListeners();
 
@@ -301,7 +300,7 @@ public class DefaultButtonModel implements ButtonModel, Serializable
    * has occurred. This happens in response to the any change to a property
    * of the model.
    */
-  public void fireStateChanged()
+  protected void fireStateChanged()
   {
     ChangeListener[] ll = getChangeListeners();
 
@@ -315,7 +314,7 @@ public class DefaultButtonModel implements ButtonModel, Serializable
    * @param stateflag DOCUMENT ME!
    * @param b DOCUMENT ME!
    */
-  protected void changeState(int stateflag, boolean b)
+  private void changeState(int stateflag, boolean b)
   {
     int oldstate = stateMask;
     int newstate;
@@ -338,16 +337,16 @@ public class DefaultButtonModel implements ButtonModel, Serializable
 
     if ((oldstate & SELECTED) == 0 && (newstate & SELECTED) == SELECTED)
       {
-	fireItemStateChanged(new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED,
-	                                   null, ItemEvent.SELECTED));
+        fireItemStateChanged(new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED,
+                                           null, ItemEvent.SELECTED));
         if (group != null)
           group.setSelected(this, true);
       }
 
     else if ((oldstate & SELECTED) == SELECTED && (newstate & SELECTED) == 0)
       {
-	fireItemStateChanged(new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED,
-	                                   null, ItemEvent.DESELECTED));
+        fireItemStateChanged(new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED,
+                                           null, ItemEvent.DESELECTED));
         if (group != null)
           group.setSelected(this, false);
       }
@@ -477,8 +476,8 @@ public class DefaultButtonModel implements ButtonModel, Serializable
   {
     if (mnemonic != key)
       {
-	mnemonic = key;
-	fireStateChanged();
+        mnemonic = key;
+        fireStateChanged();
       }
   }
 
@@ -493,8 +492,8 @@ public class DefaultButtonModel implements ButtonModel, Serializable
   {
     if (actionCommand != s)
       {
-	actionCommand = s;
-	fireStateChanged();
+        actionCommand = s;
+        fireStateChanged();
       }
   }
 
@@ -520,8 +519,8 @@ public class DefaultButtonModel implements ButtonModel, Serializable
   {
     if (group != g)
       {
-	group = g;
-	fireStateChanged();
+        group = g;
+        fireStateChanged();
       }
   }
 
