@@ -1,5 +1,5 @@
 /* DefaultDesktopManager.java --
-   Copyright (C) 2002, 2004 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -35,6 +35,7 @@ this exception to your version of the library, but you are not
 obligated to do so.  If you do not wish to do so, delete this
 exception statement from your version. */
 
+
 package javax.swing;
 
 import java.awt.Component;
@@ -90,7 +91,7 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
    */
   public DefaultDesktopManager()
   {
-  } // DefaultDesktopManager()
+  }
 
   /**
    * This method is not normally called since the user will typically add the
@@ -111,7 +112,7 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
     c.remove(frame.getDesktopIcon());
     c.add(frame);
     frame.setVisible(true);
-  } // openFrame()
+  }
 
   /**
    * This method removes the JInternalFrame and JDesktopIcon (if one is
@@ -132,7 +133,7 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
 	  c.remove(frame);
 	c.repaint();
       }
-  } // closeFrame()
+  }
 
   /**
    * This method resizes the JInternalFrame to match its parent's bounds.
@@ -171,7 +172,7 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
 	    // Do nothing.
 	  }
       }
-  } // maximizeFrame()
+  }
 
   /**
    * This method restores the JInternalFrame's bounds to what they were
@@ -188,19 +189,19 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
       p.setSelectedFrame(frame);
     else
       {
-	try
-	  {
-	    frame.setSelected(true);
-	  }
-	catch (PropertyVetoException e)
-	  {
-	    // Do nothing.
-	  }
+        try
+          {
+            frame.setSelected(true);
+          }
+        catch (PropertyVetoException e)
+          {
+            // Do nothing.
+          }
       }
 
     setBoundsForFrame(frame, normalBounds.x, normalBounds.y,
                       normalBounds.width, normalBounds.height);
-  } // minimizeFrame()
+  }
 
   /**
    * This method removes the JInternalFrame from its parent and adds its
@@ -216,34 +217,34 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
       p.setSelectedFrame(null);
     else
       {
-	try
-	  {
-	    frame.setSelected(false);
-	  }
-	catch (PropertyVetoException e)
-	  {
-	  }
+        try
+          {
+            frame.setSelected(false);
+          }
+        catch (PropertyVetoException e)
+          {
+          }
       }
 
     Container c = frame.getParent();
 
-    if (! wasIcon(frame))
+    if (!wasIcon(frame))
       {
-	Rectangle r = getBoundsForIconOf(frame);
-	icon.setBounds(r);
-	setWasIcon(frame, true);
+        Rectangle r = getBoundsForIconOf(frame);
+        icon.setBounds(r);
+        setWasIcon(frame, Boolean.TRUE);
       }
 
     if (c != null)
       {
-	if (icon != null)
-	  {
-	    c.add(icon);
-	    icon.setVisible(true);
-	  }
-	c.remove(frame);
+        if (icon != null)
+          {
+            c.add(icon);
+            icon.setVisible(true);
+          }
+        c.remove(frame);
       }
-  } // iconifyFrame()
+  }
 
   /**
    * This method removes the JInternalFrame's JDesktopIcon representation and
@@ -260,26 +261,26 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
     c.add(frame);
     frame.setVisible(true);
 
-    if (! frame.isSelected())
+    if (!frame.isSelected())
       {
-	JDesktopPane p = frame.getDesktopPane();
-	if (p != null)
-	  p.setSelectedFrame(frame);
-	else
-	  {
-	    try
-	      {
-		frame.setSelected(true);
-	      }
-	    catch (PropertyVetoException e)
-	      {
-		// Do nothing.
-	      }
-	  }
+        JDesktopPane p = frame.getDesktopPane();
+        if (p != null)
+          p.setSelectedFrame(frame);
+        else
+          {
+            try
+              {
+                frame.setSelected(true);
+              }
+            catch (PropertyVetoException e)
+              {
+                // Do nothing.
+              }
+          }
       }
 
     c.invalidate();
-  } // deiconifyFrame()
+  }
 
   /**
    * This method activates the JInternalFrame by moving it to the front and
@@ -295,17 +296,17 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
       p.setSelectedFrame(frame);
     else
       {
-	try
-	  {
-	    frame.setSelected(true);
-	  }
-	catch (PropertyVetoException e)
-	  {
-	  }
+        try
+          {
+            frame.setSelected(true);
+          }
+        catch (PropertyVetoException e)
+          {
+          }
       }
 
     frame.toFront();
-  } // activateFrame()
+  }
 
   /**
    * This method is called when the JInternalFrame loses focus.
@@ -317,20 +318,20 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
     JDesktopPane p = frame.getDesktopPane();
     if (p != null)
       {
-	if (p.getSelectedFrame() == frame)
-	  p.setSelectedFrame(null);
+        if (p.getSelectedFrame() == frame)
+          p.setSelectedFrame(null);
       }
     else
       {
-	try
-	  {
-	    frame.setSelected(false);
-	  }
-	catch (PropertyVetoException e)
-	  {
-	  }
+        try
+          {
+            frame.setSelected(false);
+          }
+        catch (PropertyVetoException e)
+          {
+          }
       }
-  } // deactivateFrame()
+  }
 
   /**
    * This method is called to indicate that the DesktopManager should prepare
@@ -354,12 +355,13 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
       currentDragMode = JDesktopPane.LIVE_DRAG_MODE;
     else
       currentDragMode = ((JDesktopPane) pane).getDragMode();
-  } // beginDraggingFrame()
+  }
 
   /**
    * This method is called to drag the JInternalFrame to a new location.
    *
    * @param component The JComponent to drag, usually a JInternalFrame.
+   *
    * @param newX The new x coordinate.
    * @param newY The new y coordinate.
    */
@@ -367,18 +369,18 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
   {
     if (currentDragMode == JDesktopPane.OUTLINE_DRAG_MODE)
       {
-	// FIXME: Do outline drag mode painting.
+        // FIXME: Do outline drag mode painting.
       }
     else
       {
-	Rectangle b = component.getBounds();
-	if (component instanceof JDesktopIcon)
-	  component.setBounds(newX, newY, b.width, b.height);
-	else
-	  setBoundsForFrame((JInternalFrame) component, newX, newY, b.width,
-	                    b.height);
+        Rectangle b = component.getBounds();
+        if (component instanceof JDesktopIcon)
+          component.setBounds(newX, newY, b.width, b.height);
+        else
+          setBoundsForFrame((JInternalFrame) component, newX, newY, b.width,
+                            b.height);
       }
-  } // dragFrame()
+  }
 
   /**
    * This method indicates that the dragging is done. Any state information
@@ -390,13 +392,13 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
   {
     if (currentDragMode == JDesktopPane.OUTLINE_DRAG_MODE)
       {
-	setBoundsForFrame((JInternalFrame) component, dragCache.x,
-	                  dragCache.y, dragCache.width, dragCache.height);
-	pane = null;
-	dragCache = null;
+        setBoundsForFrame((JInternalFrame) component, dragCache.x, dragCache.y,
+                          dragCache.width, dragCache.height);
+        pane = null;
+        dragCache = null;
       }
     component.repaint();
-  } // endDraggingFrame()
+  }
 
   /**
    * This method is called to indicate that the given JComponent will be
@@ -417,7 +419,7 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
       currentDragMode = JDesktopPane.LIVE_DRAG_MODE;
     else
       currentDragMode = ((JDesktopPane) pane).getDragMode();
-  } // beginResizingFrame()
+  }
 
   /**
    * This method resizes the give JComponent.
@@ -435,12 +437,12 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
 
     if (currentDragMode == JDesktopPane.OUTLINE_DRAG_MODE)
       {
-	// FIXME: Do outline drag painting.
+        // FIXME: Do outline drag painting.
       }
     else
       setBoundsForFrame(component, dragCache.x, dragCache.y, dragCache.width,
                         dragCache.height);
-  } // resizeFrame()
+  }
 
   /**
    * This method is called to indicate that the given JComponent has finished
@@ -453,13 +455,13 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
   {
     if (currentDragMode == JDesktopPane.OUTLINE_DRAG_MODE)
       {
-	setBoundsForFrame((JInternalFrame) component, dragCache.x,
-	                  dragCache.y, dragCache.width, dragCache.height);
-	pane = null;
-	dragCache = null;
+        setBoundsForFrame((JInternalFrame) component, dragCache.x, dragCache.y,
+                          dragCache.width, dragCache.height);
+        pane = null;
+        dragCache = null;
       }
     component.repaint();
-  } // endResizingFrame()
+  }
 
   /**
    * This method calls setBounds with the given parameters and repaints the
@@ -482,7 +484,7 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
       component.getParent().repaint();
     else
       component.repaint();
-  } // setBoundsForFrame()
+  }
 
   /**
    * This is a helper method that removes the JDesktopIcon of the given
@@ -496,7 +498,7 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
     Container c = icon.getParent();
     if (c != null && icon != null)
       c.remove(icon);
-  } // removeIconFor()
+  }
 
   /**
    * This method is called by iconifyFrame to determine the bounds of the
@@ -515,13 +517,14 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
     // The icon also must not be placed where another icon is placed 
     // (regardless whether that frame is an icon currently or not)
     JDesktopPane desktopPane = frame.getDesktopPane();
-    Rectangle paneBounds = desktopPane.getBounds();
-    Insets insets = desktopPane.getInsets();
-    Dimension pref = frame.getDesktopIcon().getPreferredSize();
 
     if (desktopPane == null)
       return frame.getDesktopIcon().getBounds();
 
+    Rectangle paneBounds = desktopPane.getBounds();
+    Insets insets = desktopPane.getInsets();
+    Dimension pref = frame.getDesktopIcon().getPreferredSize();
+
     Component[] frames = desktopPane.getComponents();
 
     int count = 0;
@@ -533,12 +536,12 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
     iconRects = new Rectangle[count];
     for (int i = 0, j = 0; i < frames.length; i++)
       if (frames[i] instanceof JDesktopIcon)
-	iconRects[--count] = frames[i].getBounds();
+        iconRects[--count] = frames[i].getBounds();
       else if (frames[i] instanceof JInternalFrame
                && ((JInternalFrame) frames[i]).getWasIcon()
                && frames[i] != frame)
-	iconRects[--count] = ((JInternalFrame) frames[i]).getDesktopIcon()
-	                      .getBounds();
+        iconRects[--count] = ((JInternalFrame) frames[i])
+                                                 .getDesktopIcon().getBounds();
 
     int startingX = insets.left;
     int startingY = paneBounds.height - insets.bottom - pref.height;
@@ -548,29 +551,29 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
 
     while (iconRects.length > 0)
       {
-	clear = true;
-	for (int i = 0; i < iconRects.length; i++)
-	  {
-	    if (iconRects[i] != null && iconRects[i].intersects(ideal))
-	      {
-		clear = false;
-		break;
-	      }
-	  }
-	if (clear)
-	  return ideal;
+        clear = true;
+        for (int i = 0; i < iconRects.length; i++)
+          {
+            if (iconRects[i] != null && iconRects[i].intersects(ideal))
+              {
+                clear = false;
+                break;
+              }
+          }
+        if (clear)
+          return ideal;
 
-	startingX += pref.width;
-	if (startingX + pref.width > paneBounds.width - insets.right)
-	  {
-	    startingX = insets.left;
-	    startingY -= pref.height;
-	  }
-	ideal.setBounds(startingX, startingY, pref.width, pref.height);
+        startingX += pref.width;
+        if (startingX + pref.width > paneBounds.width - insets.right)
+          {
+            startingX = insets.left;
+            startingY -= pref.height;
+          }
+        ideal.setBounds(startingX, startingY, pref.width, pref.height);
       }
 
     return ideal;
-  } // getBoundsForIconOf()
+  }
 
   /**
    * This method sets the bounds of the JInternalFrame right before the
@@ -582,7 +585,7 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
   protected void setPreviousBounds(JInternalFrame frame, Rectangle rect)
   {
     frame.setNormalBounds(rect);
-  } // setPreviousBounds()
+  }
 
   /**
    * This method returns the normal bounds of the JInternalFrame from before
@@ -595,7 +598,7 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
   protected Rectangle getPreviousBounds(JInternalFrame frame)
   {
     return frame.getNormalBounds();
-  } // getPreviousBounds()
+  }
 
   /**
    * This method sets the value to true if the given JInternalFrame has been
@@ -605,10 +608,10 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
    * @param value True if the JInternalFrame has been iconized and the bounds
    *        of the JDesktopIcon are valid.
    */
-  protected void setWasIcon(JInternalFrame frame, boolean value)
+  protected void setWasIcon(JInternalFrame frame, Boolean value)
   {
-    frame.setWasIcon(value, WAS_ICON_ONCE_PROPERTY);
-  } // setWasIcon()
+    frame.setWasIcon(value.booleanValue(), WAS_ICON_ONCE_PROPERTY);
+  }
 
   /**
    * This method returns true if the given JInternalFrame has been iconized
@@ -622,5 +625,5 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
   protected boolean wasIcon(JInternalFrame frame)
   {
     return frame.getWasIcon();
-  } // wasIcon()
-} // DefaultDesktopManager
+  }
+}
