@@ -52,10 +52,10 @@ import javax.swing.border.EmptyBorder;
  * @author Andrew Selkirk
  * @version 1.0
  */
-public class DefaultListCellRenderer extends JLabel implements ListCellRenderer,
-                                                               Serializable
+public class DefaultListCellRenderer extends JLabel
+  implements ListCellRenderer, Serializable
 {
-  static final long serialVersionUID = 7708947179685189462L;
+  private static final long serialVersionUID = 7708947179685189462L;
 
   public static class UIResource extends DefaultListCellRenderer
     implements javax.swing.plaf.UIResource
@@ -89,24 +89,25 @@ public class DefaultListCellRenderer extends JLabel implements ListCellRenderer,
     String s = value.toString();
     setText(s);
     setOpaque(true);
+    setHorizontalAlignment(LEFT);
 
     if (isSelected)
       {
-	setBackground(list.getSelectionBackground());
-	setForeground(list.getSelectionForeground());
+        setBackground(list.getSelectionBackground());
+        setForeground(list.getSelectionForeground());
       }
     else
       {
-	setBackground(list.getBackground());
-	setForeground(list.getForeground());
+        setBackground(list.getBackground());
+        setForeground(list.getForeground());
       }
 
     setEnabled(list.isEnabled());
     setFont(list.getFont());
 
-    // Use focusCellHighlightBorder when renderer has focus and 
+    // Use focusCellHighlightBorder when renderer has focus and
     // noFocusBorder otherwise
-    
+
     if (cellHasFocus)
       setBorder(UIManager.getBorder("List.focusCellHighlightBorder"));
     else
