@@ -1,5 +1,5 @@
 /* JComboBox.java --
-   Copyright (C) 2002, 2004 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -92,46 +92,10 @@ public class JComboBox extends JComponent implements ItemSelectable,
    * Maximum number of rows that should be visible by default  in the
    * JComboBox's popup
    */
-  public static final int DEFAULT_MAXIMUM_ROW_COUNT = 8;
+  private static final int DEFAULT_MAXIMUM_ROW_COUNT = 8;
 
   /**
-   * Fired in a PropertyChangeEvent when the 'editable' property changes.
-   */
-  public static final String EDITABLE_CHANGED_PROPERTY = "editable";
-
-  /**
-   * Fired in a PropertyChangeEvent when the 'maximumRowCount' property
-   * changes.
-   */
-  public static final String MAXIMUM_ROW_COUNT_CHANGED_PROPERTY = "maximumRowCount";
-
-  /**
-   * Fired in a PropertyChangeEvent when the 'enabled' property changes.
-   */
-  public static final String ENABLED_CHANGED_PROPERTY = "enabled";
-
-  /**
-   * Fired in a PropertyChangeEvent when the 'renderer' property changes.
-   */
-  public static final String RENDERER_CHANGED_PROPERTY = "renderer";
-
-  /**
-   * Fired in a PropertyChangeEvent when the 'editor' property changes.
-   */
-  public static final String EDITOR_CHANGED_PROPERTY = "editor";
-
-  /**
-   * Fired in a PropertyChangeEvent when the 'dataModel' property changes.
-   */
-  public static final String MODEL_CHANGED_PROPERTY = "dataModel";
-
-  /**
-   * name for the UI delegate for this combo box.
-   */
-  private static final String uiClassID = "ComboBoxUI";
-
-  /**
-   * dataModel used by JComboBox to keep track of its list data and currently
+   * Data model used by JComboBox to keep track of its list data and currently
    * selected element in the list.
    */
   protected ComboBoxModel dataModel;
@@ -221,6 +185,9 @@ public class JComboBox extends JComponent implements ItemSelectable,
   public JComboBox(Object[] itemArray)
   {
     this(new DefaultComboBoxModel(itemArray));
+    
+    if (itemArray.length > 0) 
+      setSelectedIndex(0);
   }
 
   /**
@@ -231,6 +198,9 @@ public class JComboBox extends JComponent implements ItemSelectable,
   public JComboBox(Vector itemVector)
   {
     this(new DefaultComboBoxModel(itemVector));
+
+    if (itemVector.size() > 0)
+      setSelectedIndex(0);
   }
 
   /**
@@ -298,7 +268,7 @@ public class JComboBox extends JComponent implements ItemSelectable,
    */
   public String getUIClassID()
   {
-    return uiClassID;
+    return "ComboBoxUI";
   }
 
   /**
@@ -320,16 +290,16 @@ public class JComboBox extends JComponent implements ItemSelectable,
    */
   public void setModel(ComboBoxModel newDataModel)
   {
-
     // dataModel is null if it this method is called from inside the constructors.
-    if(dataModel != null) {
-	// Prevents unneccessary updates.
-	if (dataModel == newDataModel)
-		return;
+    if (dataModel != null)
+      {
+        // Prevents unneccessary updates.
+        if (dataModel == newDataModel)
+          return;
 
-    	// Removes itself (as DataListener) from the to-be-replaced model.
-    	dataModel.removeListDataListener(this);
-    }
+        // Removes itself (as DataListener) from the to-be-replaced model.
+        dataModel.removeListDataListener(this);
+      }
     
     /* Adds itself as a DataListener to the new model.
      * It is intentioned that this operation will fail with a NullPointerException if the
@@ -342,7 +312,7 @@ public class JComboBox extends JComponent implements ItemSelectable,
     dataModel = newDataModel;
 
     // Notifies the listeners of the model change.
-    firePropertyChange(MODEL_CHANGED_PROPERTY, oldDataModel, dataModel);
+    firePropertyChange("model", oldDataModel, dataModel);
   }
 
   /**
@@ -396,8 +366,8 @@ public class JComboBox extends JComponent implements ItemSelectable,
   {
     if (isEditable != editable)
       {
-	isEditable = editable;
-	firePropertyChange(EDITABLE_CHANGED_PROPERTY, ! isEditable, isEditable);
+        isEditable = editable;
+        firePropertyChange("editable", !isEditable, isEditable);
       }
   }
 
@@ -413,10 +383,10 @@ public class JComboBox extends JComponent implements ItemSelectable,
   {
     if (maximumRowCount != rowCount)
       {
-	int oldMaximumRowCount = maximumRowCount;
-	maximumRowCount = rowCount;
-	firePropertyChange(MAXIMUM_ROW_COUNT_CHANGED_PROPERTY,
-	                   oldMaximumRowCount, maximumRowCount);
+        int oldMaximumRowCount = maximumRowCount;
+        maximumRowCount = rowCount;
+        firePropertyChange("maximumRowCount", oldMaximumRowCount,
+                           maximumRowCount);
       }
   }
 
@@ -445,10 +415,9 @@ public class JComboBox extends JComponent implements ItemSelectable,
   {
     if (renderer != aRenderer)
       {
-	ListCellRenderer oldRenderer = renderer;
-	renderer = aRenderer;
-	firePropertyChange(RENDERER_CHANGED_PROPERTY, oldRenderer,
-	                   renderer);
+        ListCellRenderer oldRenderer = renderer;
+        renderer = aRenderer;
+        firePropertyChange("renderer", oldRenderer, renderer);
       }
   }
 
@@ -483,7 +452,7 @@ public class JComboBox extends JComponent implements ItemSelectable,
     if (editor != null)
       editor.addActionListener(this);
 
-    firePropertyChange(EDITOR_CHANGED_PROPERTY, oldEditor, editor);
+    firePropertyChange("editor", oldEditor, editor);
   }
 
   /**
@@ -531,52 +500,52 @@ public class JComboBox extends JComponent implements ItemSelectable,
    */
   public void setSelectedIndex(int index)
   {
-  	if(index < -1 || index >= dataModel.getSize()) {
-  		// Fails because index is out of bounds. 
-  		throw new IllegalArgumentException("illegal index: " + index);
-  	} else {
-  		/* Selects the item at the given index or clears the selection if the
-  		 * index value is -1.
-  		 */
-		setSelectedItem((index == -1) ? null : dataModel.getElementAt(index));
-  	}
+  	if (index < -1 || index >= dataModel.getSize())
+      // Fails because index is out of bounds.
+      throw new IllegalArgumentException("illegal index: " + index);
+    else
+       // Selects the item at the given index or clears the selection if the
+       // index value is -1.
+      setSelectedItem((index == -1) ? null : dataModel.getElementAt(index));
   }
 
   /**
-   * Returns index of the item that is currently selected  in the combo box.
-   * If no item is currently selected, then -1 is returned.
-   * 
-   * <p>Note: For performance reasons you should minimize invocation of this
+   * Returns index of the item that is currently selected in the combo box. If
+   * no item is currently selected, then -1 is returned.
+   * <p>
+   * Note: For performance reasons you should minimize invocation of this
    * method. If the data model is not an instance of
-   * <code>DefaultComboBoxModel</code> the complexity is O(n) where
-   * n is the number of elements in the combo box.</p>
-   *
-   * @return int Index specifying location of the currently selected item in
-   *         the combo box or -1 if nothing is selected in the combo box.
+   * <code>DefaultComboBoxModel</code> the complexity is O(n) where n is the
+   * number of elements in the combo box.
+   * </p>
+   * 
+   * @return int Index specifying location of the currently selected item in the
+   *         combo box or -1 if nothing is selected in the combo box.
    */
   public int getSelectedIndex()
   {
     Object selectedItem = getSelectedItem();
-    
-    if (selectedItem != null) {
-	
-		if(dataModel instanceof DefaultComboBoxModel) {
-			// Uses special method of DefaultComboBoxModel to retrieve the index.
-        	  	return ((DefaultComboBoxModel) dataModel).getIndexOf(selectedItem);
-		} else {
-			// Iterates over all items to retrieve the index.
-			int size = dataModel.getSize();
-			
-	  		for(int i=0; i < size; i++) {
-	  			Object o = dataModel.getElementAt(i);
-	  			
-				// XXX: Is special handling of ComparableS neccessary?
-	  			if((selectedItem != null) ? selectedItem.equals(o) : o == null) {
-	  				return i;
-	  			}
-	  		}
-		}
-    }
+
+    if (selectedItem != null)
+      {
+        if (dataModel instanceof DefaultComboBoxModel)
+          // Uses special method of DefaultComboBoxModel to retrieve the index.
+          return ((DefaultComboBoxModel) dataModel).getIndexOf(selectedItem);
+        else
+          {
+            // Iterates over all items to retrieve the index.
+            int size = dataModel.getSize();
+
+            for (int i = 0; i < size; i++)
+              {
+                Object o = dataModel.getElementAt(i);
+
+                // XXX: Is special handling of ComparableS neccessary?
+                if ((selectedItem != null) ? selectedItem.equals(o) : o == null)
+                  return i;
+              }
+          }
+      }
 
     // returns that no item is currently selected
     return -1;
@@ -601,11 +570,12 @@ public class JComboBox extends JComponent implements ItemSelectable,
    */
   public void addItem(Object element)
   {
-  	if(dataModel instanceof MutableComboBoxModel) {
-		((MutableComboBoxModel) dataModel).addElement(element);
-  	} else {
-  		throw new RuntimeException("Unable to add the item because the data model it is not an instance of MutableComboBoxModel.");
-  	}
+  	if (dataModel instanceof MutableComboBoxModel)
+      ((MutableComboBoxModel) dataModel).addElement(element);
+    else
+      throw new RuntimeException("Unable to add the item because the data "
+                                 + "model it is not an instance of "
+                                 + "MutableComboBoxModel.");
   }
 
   /**
@@ -618,11 +588,12 @@ public class JComboBox extends JComponent implements ItemSelectable,
    */
   public void insertItemAt(Object element, int index)
   {
-	if(dataModel instanceof MutableComboBoxModel) {
-		((MutableComboBoxModel) dataModel).insertElementAt(element, index);
-	} else {
-		throw new RuntimeException("Unable to insert the item because the data model it is not an instance of MutableComboBoxModel.");
-	}
+	if (dataModel instanceof MutableComboBoxModel)
+      ((MutableComboBoxModel) dataModel).insertElementAt(element, index);
+    else
+      throw new RuntimeException("Unable to insert the item because the data "
+                                 + "model it is not an instance of "
+                                 + "MutableComboBoxModel.");
   }
 
   /**
@@ -634,11 +605,12 @@ public class JComboBox extends JComponent implements ItemSelectable,
    */
   public void removeItem(Object element)
   {
-	if(dataModel instanceof MutableComboBoxModel) {
-		((MutableComboBoxModel) dataModel).removeElement(element);
-	} else {
-		throw new RuntimeException("Unable to remove the item because the data model it is not an instance of MutableComboBoxModel.");
-	}
+	if (dataModel instanceof MutableComboBoxModel)
+      ((MutableComboBoxModel) dataModel).removeElement(element);
+    else
+      throw new RuntimeException("Unable to remove the item because the data "
+                                 + "model it is not an instance of "
+                                 + "MutableComboBoxModel.");
   }
 
   /**
@@ -651,40 +623,40 @@ public class JComboBox extends JComponent implements ItemSelectable,
    */
   public void removeItemAt(int index)
   {
-	if(dataModel instanceof MutableComboBoxModel) {
-		((MutableComboBoxModel) dataModel).removeElementAt(index);
-	} else {
-		throw new RuntimeException("Unable to remove the item because the data model it is not an instance of MutableComboBoxModel.");
-	}
+    if (dataModel instanceof MutableComboBoxModel)
+      ((MutableComboBoxModel) dataModel).removeElementAt(index);
+    else
+      throw new RuntimeException("Unable to remove the item because the data "
+                                 + "model it is not an instance of "
+                                 + "MutableComboBoxModel.");
   }
 
   /**
    * This method removes all elements from this JComboBox.
-   * <p>A <code>RuntimeException</code> is thrown if the data model is not
-   * an instance of {@link MutableComboBoxModel}.</p>
-   * 
+   * <p>
+   * A <code>RuntimeException</code> is thrown if the data model is not an
+   * instance of {@link MutableComboBoxModel}.
+   * </p>
    */
   public void removeAllItems()
   {
-    if (dataModel instanceof DefaultComboBoxModel) {
-    	// Uses special method if we have a DefaultComboBoxModel.
-	((DefaultComboBoxModel) dataModel).removeAllElements();
-    } else if(dataModel instanceof MutableComboBoxModel){
-    	// Iterates over all items and removes each.
-    	MutableComboBoxModel mcbm = (MutableComboBoxModel) dataModel;
+    if (dataModel instanceof DefaultComboBoxModel)
+      // Uses special method if we have a DefaultComboBoxModel.
+      ((DefaultComboBoxModel) dataModel).removeAllElements();
+    else if (dataModel instanceof MutableComboBoxModel)
+      {
+        // Iterates over all items and removes each.
+        MutableComboBoxModel mcbm = (MutableComboBoxModel) dataModel;
 
-	/* We intentionally remove the items backwards to support
-	 * models which shift their content to the beginning (e.g.
-	 * linked lists) 
-	 */     	
-    	for(int i=mcbm.getSize()-1; i >= 0; i--) {
-    		mcbm.removeElementAt(i);
-    	}
-    	
-    } else {
-	throw new RuntimeException("Unable to remove the items because the data model it is not an instance of MutableComboBoxModel.");
-    }
-      
+         // We intentionally remove the items backwards to support models which
+         // shift their content to the beginning (e.g. linked lists)
+        for (int i = mcbm.getSize() - 1; i >= 0; i--)
+          mcbm.removeElementAt(i);
+      }
+    else
+      throw new RuntimeException("Unable to remove the items because the data "
+                                 +"model it is not an instance of "
+                                 + "MutableComboBoxModel.");
   }
 
   /**
@@ -785,13 +757,13 @@ public class JComboBox extends JComponent implements ItemSelectable,
   {
     if (a == null)
       {
-	setEnabled(true);
-	setToolTipText(null);
+        setEnabled(true);
+        setToolTipText(null);
       }
     else
       {
-	setEnabled(a.isEnabled());
-	setToolTipText((String) (a.getValue(Action.SHORT_DESCRIPTION)));
+        setEnabled(a.isEnabled());
+        setToolTipText((String) (a.getValue(Action.SHORT_DESCRIPTION)));
       }
   }
 
@@ -801,18 +773,18 @@ public class JComboBox extends JComponent implements ItemSelectable,
    *
    * @param action action to listen to for property changes
    *
-   * @return $PropertyChangeListener$ Listener that listens to changes in
+   * @return a PropertyChangeListener that listens to changes in
    *         action properties.
    */
   protected PropertyChangeListener createActionPropertyChangeListener(Action action)
   {
     return new PropertyChangeListener()
       {
-	public void propertyChange(PropertyChangeEvent e)
-	{
-	  Action act = (Action) (e.getSource());
-	  configurePropertiesFromAction(act);
-	}
+        public void propertyChange(PropertyChangeEvent e)
+        {
+          Action act = (Action) (e.getSource());
+          configurePropertiesFromAction(act);
+        }
       };
   }
 
@@ -966,9 +938,8 @@ public class JComboBox extends JComponent implements ItemSelectable,
     boolean oldEnabled = super.isEnabled();
     if (enabled != oldEnabled)
       {
-	super.setEnabled(enabled);
-	firePropertyChange(ENABLED_CHANGED_PROPERTY, oldEnabled,
-	                   (boolean) enabled);
+        super.setEnabled(enabled);
+        firePropertyChange("enabled", oldEnabled, enabled);
       }
   }
 
