@@ -46,6 +46,7 @@ import java.awt.Dimension;
 import java.awt.FlowLayout;
 import java.awt.Font;
 import java.awt.Graphics;
+import java.awt.Graphics2D;
 import java.awt.Image;
 import java.awt.Insets;
 import java.awt.Point;
@@ -73,10 +74,12 @@ import java.util.Locale;
 
 import javax.accessibility.Accessible;
 import javax.accessibility.AccessibleContext;
+import javax.accessibility.AccessibleExtendedComponent;
 import javax.accessibility.AccessibleKeyBinding;
 import javax.accessibility.AccessibleRole;
 import javax.accessibility.AccessibleStateSet;
 import javax.swing.border.Border;
+import javax.swing.event.AncestorEvent;
 import javax.swing.event.AncestorListener;
 import javax.swing.event.EventListenerList;
 import javax.swing.event.SwingPropertyChangeSupport;
@@ -98,11 +101,11 @@ public abstract class JComponent extends Container implements Serializable
   /** 
    * Accessibility support is currently missing.
    */
-
   protected AccessibleContext accessibleContext;
 
   public abstract class AccessibleJComponent 
     extends AccessibleAWTContainer
+    implements AccessibleExtendedComponent
   {
     protected class AccessibleFocusHandler 
       implements FocusListener
@@ -161,7 +164,6 @@ public abstract class JComponent extends Container implements Serializable
    */
   Dimension maximumSize;
 
-
   /**
    * A value between 0.0 and 1.0 indicating the preferred horizontal
    * alignment of the component, relative to its siblings. The values
@@ -345,7 +347,6 @@ public abstract class JComponent extends Container implements Serializable
    */
   private static final Object paintLock = new Object();
 
-
   /**
    * The default locale of the component.
    * 
@@ -388,7 +389,6 @@ public abstract class JComponent extends Container implements Serializable
    */
   public static final int WHEN_IN_FOCUSED_WINDOW = 2;
 
-
   /**
    * Creates a new <code>JComponent</code> instance.
    */
@@ -399,6 +399,7 @@ public abstract class JComponent extends Container implements Serializable
     setDropTarget(new DropTarget());
     defaultLocale = Locale.getDefault();
     debugGraphicsOptions = DebugGraphics.NONE_OPTION;
+    setRequestFocusEnabled(true);
   }
 
   /**
@@ -609,12 +610,12 @@ public abstract class JComponent extends Container implements Serializable
    * @param property The property to return the listeners of
    *
    * @return The set of <code>PropertyChangeListener</code> objects in 
-   * {@link #changeSupport} registered to listen on the specified propert
+   *     {@link #changeSupport} registered to listen on the specified property
    */
   public PropertyChangeListener[] getPropertyChangeListeners(String property)
   {
     return changeSupport == null ? new PropertyChangeListener[0]
-                                 : changeSupport.getPropertyChangeListeners(property);
+                          : changeSupport.getPropertyChangeListeners(property);
   }
 
   /**
@@ -752,9 +753,9 @@ public abstract class JComponent extends Container implements Serializable
     throws PropertyVetoException
   {
     VetoableChangeListener[] listeners = getVetoableChangeListeners();
-    
+
     PropertyChangeEvent evt = new PropertyChangeEvent(this, propertyName, oldValue, newValue);
-    
+
     for (int i = 0; i < listeners.length; i++)
       listeners[i].vetoableChange(evt);
   }
@@ -769,7 +770,6 @@ public abstract class JComponent extends Container implements Serializable
     return null;
   }
 
-
   /**
    * Get the value of the {@link #alignmentX} property.
    *
@@ -872,13 +872,11 @@ public abstract class JComponent extends Container implements Serializable
    */
   protected Graphics getComponentGraphics(Graphics g)
   {    
-    Graphics g2 = g.create();
-    g2.setFont(this.getFont());
-    g2.setColor(this.getForeground());
-    return g2;
+    g.setFont (this.getFont());
+    g.setColor (this.getForeground());
+    return g;
   }
 
-
   /**
    * Get the value of the {@link #debugGraphicsOptions} property.
    *
@@ -1020,17 +1018,26 @@ public abstract class JComponent extends Container implements Serializable
    */
   public Dimension getPreferredSize()
   {
+    Dimension prefSize = null;
     if (preferredSize != null)
-      return preferredSize;
+      prefSize = preferredSize;
 
-    if (ui != null)
+    else if (ui != null)
       {
         Dimension s = ui.getPreferredSize(this);
         if (s != null)
-          return s;
+          prefSize = s;
       }
-    Dimension p = super.getPreferredSize();
-    return p;
+
+    if (prefSize == null)
+      prefSize = super.getPreferredSize();
+    // make sure that prefSize is not smaller than minSize
+    if (minimumSize != null && prefSize != null
+        && (minimumSize.width > prefSize.width
+            || minimumSize.height > prefSize.height))
+        prefSize = new Dimension(Math.max(minimumSize.width, prefSize.width),
+                                 Math.max(minimumSize.height, prefSize.height));
+    return prefSize;
   }
 
   /**
@@ -1137,10 +1144,10 @@ public abstract class JComponent extends Container implements Serializable
    */
   public JToolTip createToolTip()
   {
-        JToolTip toolTip = new JToolTip();
-        toolTip.setComponent(this);
-        toolTip.setTipText(toolTipText);
-    
+    JToolTip toolTip = new JToolTip();
+    toolTip.setComponent(this);
+    toolTip.setTipText(toolTipText);
+
     return toolTip;
   }
 
@@ -1151,7 +1158,7 @@ public abstract class JComponent extends Container implements Serializable
    * @param event The event the tooltip is being presented in response to
    *
    * @return The point at which to display a tooltip, or <code>null</code>
-   * if swing is to choose a default location.
+   *     if swing is to choose a default location.
    */
   public Point getToolTipLocation(MouseEvent event)
   {
@@ -1173,14 +1180,14 @@ public abstract class JComponent extends Container implements Serializable
       toolTipText = null;
       return;
     }
-                
+
     // XXX: The tip text doesn't get updated unless you set it to null
     // and then to something not-null. This is consistent with the behaviour
     // of Sun's ToolTipManager.
-                        
+
     String oldText = toolTipText;
     toolTipText = text;
-                
+
     if (oldText == null)
       ToolTipManager.sharedInstance().registerComponent(this);
   }
@@ -1311,10 +1318,10 @@ public abstract class JComponent extends Container implements Serializable
    * <code>CTRL+TAB</code> and <code>CTRL+SHIFT+TAB</code>.
    *
    * @return <code>true</code> if you want this component to manage its own
-   * focus, otherwise (by default) <code>false</code>
+   *     focus, otherwise (by default) <code>false</code>
    *
    * @deprecated 1.4 Use {@link Component.setFocusTraversalKeys(int,Set)} and
-   * {@link Container.setFocusCycleRoot(boolean)} instead
+   *     {@link Container.setFocusCycleRoot(boolean)} instead
    */
   public boolean isManagingFocus()
   {
@@ -1401,8 +1408,8 @@ public abstract class JComponent extends Container implements Serializable
    * one of these methods rather than <code>paint</code>.</p>
    *
    * <p>For more details on the painting sequence, see <a
-   * href="http://java.sun.com/products/jfc/tsc/articles/painting/index.html">this
-   * article</a>.</p>
+   * href="http://java.sun.com/products/jfc/tsc/articles/painting/index.html">
+   * this article</a>.</p>
    *
    * @param g The graphics context to paint with
    *
@@ -1493,7 +1500,14 @@ public abstract class JComponent extends Container implements Serializable
   protected void paintComponent(Graphics g)
   {
     if (ui != null)
-      ui.update(g, this);
+      {
+        Graphics g2 = g;
+        if (!(g instanceof Graphics2D))
+          g2 = g.create();
+        ui.update(getComponentGraphics(g2), this);
+        if (!(g instanceof Graphics2D))
+          g2.dispose();
+      }
   }
 
   /**
@@ -1592,7 +1606,6 @@ public abstract class JComponent extends Container implements Serializable
    * sun-private solution so I will only immitate it as much as it matters
    * to external observers.
    */
-
   private static class ActionListenerProxy
     extends AbstractAction
   {
@@ -1625,12 +1638,12 @@ public abstract class JComponent extends Container implements Serializable
    *
    * @param anAction The action to be registered
    * @param aCommand The command to deliver in the delivered {@link
-   * java.awt.ActionEvent}
+   *     java.awt.ActionEvent}
    * @param aKeyStroke The keystroke to register on
    * @param aCondition One of the values {@link #UNDEFINED_CONDITION},
-   * {@link #WHEN_ANCESTOR_OF_FOCUSED_COMPONENT}, {@link #WHEN_FOCUSED}, or
-   * {@link #WHEN_IN_FOCUSED_WINDOW}, indicating the condition which must
-   * be met for the action to be fired
+   *     {@link #WHEN_ANCESTOR_OF_FOCUSED_COMPONENT}, {@link #WHEN_FOCUSED}, or
+   *     {@link #WHEN_IN_FOCUSED_WINDOW}, indicating the condition which must
+   *     be met for the action to be fired
    *
    * @see #unregisterKeyboardAction
    * @see #getConditionForKeystroke
@@ -1644,8 +1657,6 @@ public abstract class JComponent extends Container implements Serializable
     getInputMap(cond).put(stroke, new ActionListenerProxy(act, cmd));
   }
 
-
-
   public final void setInputMap(int condition, InputMap map)
   {
     enableEvents(AWTEvent.KEY_EVENT_MASK);
@@ -1719,11 +1730,11 @@ public abstract class JComponent extends Container implements Serializable
    * @param aKeyStroke The keystroke to return the condition of
    *
    * @return One of the values {@link #UNDEFINED_CONDITION}, {@link
-   * #WHEN_ANCESTOR_OF_FOCUSED_COMPONENT}, {@link #WHEN_FOCUSED}, or {@link
-   * #WHEN_IN_FOCUSED_WINDOW}
+   *     #WHEN_ANCESTOR_OF_FOCUSED_COMPONENT}, {@link #WHEN_FOCUSED}, or {@link
+   *     #WHEN_IN_FOCUSED_WINDOW}
    *
    * @deprecated As of 1.3 KeyStrokes can be registered with multiple
-   * simultaneous conditions.
+   *     simultaneous conditions.
    *
    * @see #registerKeyboardAction   
    * @see #unregisterKeyboardAction   
@@ -1779,11 +1790,13 @@ public abstract class JComponent extends Container implements Serializable
    * the swing {@link InputMap} / {@link ActionMap} system.
    *
    * See <a
-   * href="http://java.sun.com/products/jfc/tsc/special_report/kestrel/keybindings.html">this
-   * report</a> for more details, it's somewhat complex.
+   * href="http://java.sun.com/products/jfc/tsc/special_report/kestrel/keybindings.html">
+   * this report</a> for more details, it's somewhat complex.
    */
   protected void processKeyEvent(KeyEvent e)
   {
+    // let the AWT event processing send KeyEvents to registered listeners
+    super.processKeyEvent(e);
     processComponentKeyEvent(e);
 
     // FIXME: this needs to be elaborated significantly, to do all the
@@ -1831,6 +1844,7 @@ public abstract class JComponent extends Container implements Serializable
    */
   public void unregisterKeyboardAction(KeyStroke aKeyStroke)
   {
+    // FIXME: Must be implemented.
   }
 
 
@@ -1853,7 +1867,6 @@ public abstract class JComponent extends Container implements Serializable
       actionMap.clear();
   }
 
-
   /**
    * Mark the described region of this component as dirty in the current
    * {@link RepaintManager}. This will queue an asynchronous repaint using
@@ -1894,8 +1907,8 @@ public abstract class JComponent extends Container implements Serializable
    *
    * @return The result of {@link #requestFocus}
    *
-   * @deprecated Use {@link #requestFocus()} on the default component provided from
-   * the {@link FocusTraversalPolicy} instead.
+   * @deprecated Use {@link #requestFocus()} on the default component provided
+   *     from the {@link FocusTraversalPolicy} instead.
    */
   public boolean requestDefaultFocus()
   {
@@ -2139,8 +2152,6 @@ public abstract class JComponent extends Container implements Serializable
     boolean oldOpaque = opaque;
     opaque = isOpaque;
     firePropertyChange("opaque", oldOpaque, opaque);
-    revalidate();
-    repaint();
   }
 
   /**
@@ -2276,4 +2287,422 @@ public abstract class JComponent extends Container implements Serializable
                        ! verifyInputWhenFocusTarget,
                        verifyInputWhenFocusTarget);
   }
+
+  /**
+   * Requests that this component gets the input focus if the
+   * requestFocusEnabled property is set to <code>true</code>.
+   * This also means that this component's top-level window becomes
+   * the focused window, if that is not already the case.
+   *
+   * The preconditions that have to be met to become a focus owner is that
+   * the component must be displayable, visible and focusable.
+   *
+   * Note that this signals only a request for becoming focused. There are
+   * situations in which it is not possible to get the focus. So developers
+   * should not assume that the component has the focus until it receives
+   * a {@link java.awt.event.FocusEvent} with a value of
+   * {@link java.awt.event.FocusEvent.FOCUS_GAINED}.
+   *
+   * @see {@link Component#requestFocus()}
+   */
+  public void requestFocus()
+  {
+    if (isRequestFocusEnabled())
+      super.requestFocus();
+  }
+
+  /**
+   * This method is overridden to make it public so that it can be used
+   * by look and feel implementations.
+   *
+   * You should not use this method directly. Instead you are strongly
+   * encouraged to call {@link #requestFocus} or {@link #requestFocusInWindow}
+   * instead.
+   *
+   * @param temporary if the focus change is temporary
+   *
+   * @return <code>false</code> if the focus change request will definitly
+   *     fail, <code>true</code> if it will likely succeed
+   *
+   * @see {@link Component#requestFocus(boolean)}
+   *
+   * @since 1.4
+   */
+  public boolean requestFocus(boolean temporary)
+  {
+    return super.requestFocus(temporary);
+  }
+
+  /**
+   * Requests that this component gets the input focus if the top level
+   * window that contains this component has the focus and the
+   * requestFocusEnabled property is set to <code>true</code>.
+   *
+   * The preconditions that have to be met to become a focus owner is that
+   * the component must be displayable, visible and focusable.
+   *
+   * Note that this signals only a request for becoming focused. There are
+   * situations in which it is not possible to get the focus. So developers
+   * should not assume that the component has the focus until it receives
+   * a {@link java.awt.event.FocusEvent} with a value of
+   * {@link java.awt.event.FocusEvent.FOCUS_GAINED}.
+   *
+   * @return <code>false</code> if the focus change request will definitly
+   *     fail, <code>true</code> if it will likely succeed
+   *
+   * @see {@link Component#requestFocusInWindow()}
+   */
+  public boolean requestFocusInWindow()
+  {
+    if (isRequestFocusEnabled())
+      return super.requestFocusInWindow();
+    else
+      return false;
+  }
+
+  /**
+   * This method is overridden to make it public so that it can be used
+   * by look and feel implementations.
+   *
+   * You should not use this method directly. Instead you are strongly
+   * encouraged to call {@link #requestFocus} or {@link #requestFocusInWindow}
+   * instead.
+   *
+   * @param temporary if the focus change is temporary
+   *
+   * @return <code>false</code> if the focus change request will definitly
+   *     fail, <code>true</code> if it will likely succeed
+   *
+   * @see {@link Component#requestFocus(boolean)}
+   *
+   * @since 1.4
+   */
+  public boolean requestFocusInWindow(boolean temporary)
+  {
+    return super.requestFocusInWindow(temporary);
+  }
+
+  /**
+   * Receives notification if this component is added to a parent component.
+   *
+   * Notification is sent to all registered AncestorListeners about the
+   * new parent.
+   *
+   * This method sets up ActionListeners for all registered KeyStrokes of
+   * this component in the chain of parent components.
+   *
+   * A PropertyChange event is fired to indicate that the ancestor property
+   * has changed.
+   *
+   * This method is used internally and should not be used in applications.
+   */
+  public void addNotify()
+  {
+    super.addNotify();
+
+    // let parents inherit the keybord mapping
+    InputMap input = getInputMap();
+    ActionMap actions = getActionMap();
+
+    Container parent = getParent();
+    while ((parent != null) && (parent instanceof JComponent))
+      {
+        JComponent jParent = (JComponent) parent;
+        InputMap parentInput = jParent.getInputMap();
+        ActionMap parentAction = jParent.getActionMap();
+
+        KeyStroke[] ikeys = input.keys();
+        for (int i = 0; i < ikeys.length; i++)
+          {
+            Object o = input.get(ikeys[i]);
+            parentInput.put(ikeys[i], o);
+          }
+
+        Object[] akeys = actions.keys();
+        for (int i = 0; i < akeys.length; i++)
+          {
+            Action a = actions.get(akeys[i]);
+            parentAction.put(akeys[i], a);
+          }
+
+        parent = jParent.getParent();
+      }
+
+    // notify ancestor listeners
+    AncestorListener[] ls = getAncestorListeners();
+    AncestorEvent ev = new AncestorEvent(this, AncestorEvent.ANCESTOR_ADDED,
+                                         this, parent);
+    for (int i = 0; i < ls.length; i++)
+      {
+        ls[i].ancestorAdded(ev);
+      }
+
+    // fire property change event for 'ancestor'
+    firePropertyChange("ancestor", null, parent);
+  }
+
+  /**
+   * Receives notification that this component no longer has a parent.
+   *
+   * This method sends an AncestorEvent to all registered AncestorListeners,
+   * notifying them that the parent is gone.
+   *
+   * The keybord actions of this component are removed from the parent and
+   * its ancestors.
+   *
+   * A PropertyChangeEvent is fired to indicate that the 'ancestor' property
+   * has changed.
+   *
+   * This method is called before the component is actually removed from
+   * its parent, so the parent is still visible through {@link #getParent}.
+   */
+  public void removeNotify()
+  {
+    super.removeNotify();
+
+    // let parents inherit the keybord mapping
+    InputMap input = getInputMap();
+    ActionMap actions = getActionMap();
+
+    Container parent = getParent();
+    while ((parent != null) && (parent instanceof JComponent))
+      {
+        JComponent jParent = (JComponent) parent;
+        InputMap parentInput = jParent.getInputMap();
+        ActionMap parentAction = jParent.getActionMap();
+
+        KeyStroke[] ikeys = input.allKeys();
+        for (int i = 0; i < ikeys.length; i++)
+          {
+            parentInput.remove(ikeys[i]);
+          }
+
+        Object[] akeys = actions.allKeys();
+        for (int i = 0; i < akeys.length; i++)
+          {
+            parentAction.remove(akeys[i]);
+          }
+
+        parent = jParent.getParent();
+      }
+
+    // notify ancestor listeners
+    AncestorListener[] ls = getAncestorListeners();
+    AncestorEvent ev = new AncestorEvent(this, AncestorEvent.ANCESTOR_ADDED,
+                                         this, parent);
+    for (int i = 0; i < ls.length; i++)
+      {
+        ls[i].ancestorAdded(ev);
+      }
+
+    // fire property change event for 'ancestor'
+    firePropertyChange("ancestor", parent, null);
+  }
+
+  /**
+   * Returns <code>true</code> if the coordinates (x, y) lie within
+   * the bounds of this component and <code>false</code> otherwise.
+   * x and y are relative to the coordinate space of the component.
+   *
+   * @param x the X coordinate of the point to check
+   * @param y the Y coordinate of the point to check
+   *
+   * @return <code>true</code> if the specified point lies within the bounds
+   *     of this component, <code>false</code> otherwise
+   */
+  public boolean contains(int x, int y)
+  {
+    if (ui == null)
+      return super.contains(x, y);
+    else
+      return ui.contains(this, x, y);
+  }
+
+  /**
+   * Disables this component.
+   *
+   * @deprecated replaced by {@link #setEnabled(boolean)}
+   */
+  public void disable()
+  {
+    super.disable();
+  }
+
+  /**
+   * Enables this component.
+   *
+   * @deprecated replaced by {@link #setEnabled(boolean)}
+   */
+  public void enable()
+  {
+    super.enable();
+  }
+
+  /**
+   * Returns the Graphics context for this component. This can be used
+   * to draw on a component.
+   *
+   * @return the Graphics context for this component
+   */
+  public Graphics getGraphics()
+  {
+    return super.getGraphics();
+  }
+
+  /**
+   * Returns the X coordinate of the upper left corner of this component.
+   * Prefer this method over {@link #getBounds} or {@link #getLocation}
+   * because it does not cause any heap allocation.
+   *
+   * @return the X coordinate of the upper left corner of the component
+   */
+  public int getX()
+  {
+    return super.getX();
+  }
+
+  /**
+   * Returns the Y coordinate of the upper left corner of this component.
+   * Prefer this method over {@link #getBounds} or {@link #getLocation}
+   * because it does not cause any heap allocation.
+   *
+   * @return the Y coordinate of the upper left corner of the component
+   */
+  public int getY()
+  {
+    return super.getY();
+  }
+
+  /**
+   * Returns the height of this component. Prefer this method over
+   * {@link #getBounds} or {@link #getSize} because it does not cause
+   * any heap allocation.
+   *
+   * @return the height of the component
+   */
+  public int getHeight()
+  {
+    return super.getHeight();
+  }
+
+  /**
+   * Returns the width of this component. Prefer this method over
+   * {@link #getBounds} or {@link #getSize} because it does not cause
+   * any heap allocation.
+   *
+   * @return the width of the component
+   */
+  public int getWidth()
+  {
+    return super.getWidth();
+  }
+
+  /**
+   * Return all <code>PropertyChangeListener</code> objects registered.
+   *
+   * @return The set of <code>PropertyChangeListener</code> objects
+   */
+  public PropertyChangeListener[] getPropertyChangeListeners()
+  {
+    if (changeSupport == null)
+      return new PropertyChangeListener[0];
+    else
+      return changeSupport.getPropertyChangeListeners();
+  }
+
+  /**
+   * Prints this component to the given Graphics context. A call to this
+   * method results in calls to the methods {@link #printComponent},
+   * {@link #printBorder} and {@link printChildren} in this order.
+   *
+   * Double buffering is temporarily turned off so the painting goes directly
+   * to the supplied Graphics context.
+   *
+   * @param g the Graphics context to print onto
+   */
+  public void print(Graphics g)
+  {
+    boolean doubleBufferState = isDoubleBuffered();
+    setDoubleBuffered(false);
+    printComponent(g);
+    printBorder(g);
+    printChildren(g);
+    setDoubleBuffered(doubleBufferState);
+  }
+
+  /**
+   * Prints this component to the given Graphics context. This invokes
+   * {@link #print}.
+   *
+   * @param g the Graphics context to print onto
+   */
+  public void printAll(Graphics g)
+  {
+    print(g);
+  }
+
+  /**
+   * Prints this component to the specified Graphics context. The default
+   * behaviour is to invoke {@link #paintComponent}. Override this
+   * if you want special behaviour for printing.
+   *
+   * @param g the Graphics context to print onto
+   *
+   * @since 1.3
+   */
+  public void printComponent(Graphics g)
+  {
+    paintComponent(g);
+  }
+
+  /**
+   * Print this component's children to the specified Graphics context.
+   * The default behaviour is to invoke {@link #paintChildren}. Override this
+   * if you want special behaviour for printing.
+   *
+   * @param g the Graphics context to print onto
+   *
+   * @since 1.3
+   */
+  public void printChildren(Graphics g)
+  {
+    paintChildren(g);
+  }
+
+  /**
+   * Print this component's border to the specified Graphics context.
+   * The default behaviour is to invoke {@link #paintBorder}. Override this
+   * if you want special behaviour for printing.
+   *
+   * @param g the Graphics context to print onto
+   *
+   * @since 1.3
+   */
+  public void printBorder(Graphics g)
+  {
+    paintBorder(g);
+  }
+
+  /**
+   * Processes mouse motion event, like dragging and moving.
+   *
+   * @param ev the MouseEvent describing the mouse motion
+   */
+  protected void processMouseMotionEvent(MouseEvent ev)
+  {
+    super.processMouseMotionEvent(ev);
+  }
+
+  /**
+   * Moves and resizes the component.
+   *
+   * @param x the new horizontal location
+   * @param y the new vertial location
+   * @param w the new width
+   * @param h the new height
+   */
+  public void reshape(int x, int y, int w, int h)
+  {
+    super.reshape(x, y, w, h);
+  }
 }
