@@ -35,9 +35,9 @@ this exception to your version of the library, but you are not
 obligated to do so.  If you do not wish to do so, delete this
 exception statement from your version. */
 
+
 package javax.swing;
 
-import java.awt.BorderLayout;
 import java.awt.Component;
 import java.awt.Container;
 import java.awt.Dialog;
@@ -45,14 +45,13 @@ import java.awt.Dimension;
 import java.awt.Frame;
 import java.awt.Graphics;
 import java.awt.GraphicsConfiguration;
-import java.awt.LayoutManager;
-import java.awt.event.KeyEvent;
-import java.awt.event.WindowEvent;
 import java.awt.IllegalComponentStateException;
+import java.awt.LayoutManager;
+import java.awt.event.WindowEvent;
+
 import javax.accessibility.Accessible;
 import javax.accessibility.AccessibleContext;
 
-
 /**
  * Unlike JComponent derivatives, JDialog inherits from java.awt.Dialog. But
  * also lets a look-and-feel component to its work.
@@ -70,17 +69,27 @@ public class JDialog extends Dialog implements Accessible, WindowConstants,
   /** The single RootPane in the Dialog. */
   protected JRootPane rootPane;
 
-  /** Whether checking is enabled on the RootPane */
-  protected boolean rootPaneCheckingEnabled = true;
+  /**
+   * Whether checking is enabled on the RootPane.
+   *
+   * @specnote Should be false to comply with J2SE 5.0
+   */ 
+  protected boolean rootPaneCheckingEnabled = false;
 
   /** The default action taken when closed. */
   private int close_action = HIDE_ON_CLOSE;
   
   /** Whether JDialogs are decorated by the Look and Feel. */
-  private static boolean decorated = false;
+  private static boolean decorated;
 
   /**
-   * Creates a new non-modal JDialog with no title 
+   * Whether we're in the init stage or not.
+   * If so, adds and layouts are for top-level, otherwise they're for the
+   * content pane
+   */
+  private boolean initStageDone = false;
+
+  /* Creates a new non-modal JDialog with no title 
    * using a shared Frame as the owner.
    */
   public JDialog()
@@ -225,12 +234,12 @@ public class JDialog extends Dialog implements Accessible, WindowConstants,
   protected void dialogInit()
   {
     // FIXME: Do a check on GraphicsEnvironment.isHeadless()
-    setRootPaneCheckingEnabled(false);
-    setLocale(JComponent.getDefaultLocale());       
-    getRootPane(); // will do set/create  
-    setRootPaneCheckingEnabled(true);    
+    setLocale(JComponent.getDefaultLocale());
+    getRootPane(); // Will do set/create.
     invalidate();
-
+    // Now that initStageDone is true, adds and layouts apply to contentPane,
+    // not top-level.
+    initStageDone = true;
   }
 
   /**
@@ -299,9 +308,17 @@ public class JDialog extends Dialog implements Accessible, WindowConstants,
    */
   public void setLayout(LayoutManager manager)
   {
-    if (isRootPaneCheckingEnabled())
-      throw new Error("rootPaneChecking is enabled - cannot set layout.");
-    super.setLayout(manager);
+    // Check if we're in initialization stage. If so, call super.setLayout
+    // otherwise, valid calls go to the content pane.
+    if (initStageDone)
+      {
+        if (isRootPaneCheckingEnabled())
+          throw new Error("Cannot set top-level layout.  Use"
+                           + " getConentPane().setLayout instead.");
+          getContentPane().setLayout(manager);
+      }
+    else
+      super.setLayout(manager);
   }
 
   /**
@@ -421,9 +438,17 @@ public class JDialog extends Dialog implements Accessible, WindowConstants,
    */
   protected void addImpl(Component comp, Object constraints, int index)
   {
-    if (isRootPaneCheckingEnabled())
-      throw new Error("rootPaneChecking is enabled - adding components disallowed.");
-    super.addImpl(comp, constraints, index);
+    // If we're adding the rootPane (initialization stages) use super.add.
+    // Otherwise pass the add onto the content pane.
+    if (comp == rootPane)
+      super.addImpl(comp, constraints, index);
+    else
+      {
+        if (isRootPaneCheckingEnabled())
+          throw new Error("Do not add directly to JDialog."
+                          + " Use getContentPane().add instead.");
+        getContentPane().add(comp, constraints, index);
+      }
   }
 
   /**
@@ -433,7 +458,8 @@ public class JDialog extends Dialog implements Accessible, WindowConstants,
    */
   public void remove(Component comp)
   {
-    // The path changes if the component == root.
+    // If we're removing the root pane, use super.remove. Otherwise
+    // pass it on to the content pane instead.
     if (comp == rootPane)
       super.remove(rootPane);
     else 
@@ -521,12 +547,12 @@ public class JDialog extends Dialog implements Accessible, WindowConstants,
    */
   public void setDefaultCloseOperation(int operation)
   {
-    if (operation == DO_NOTHING_ON_CLOSE ||
-    	operation == HIDE_ON_CLOSE ||
-	operation == DISPOSE_ON_CLOSE)
-      close_action = operation;
-    else
-      throw new IllegalArgumentException("Default close operation must be one of DO_NOTHING_ON_CLOSE, HIDE_ON_CLOSE, or DISPOSE_ON_CLOSE");
+    /* Reference implementation allows invalid operations
+       to be specified.  If so, getDefaultCloseOperation
+       must return the invalid code, and the behaviour
+       defaults to DO_NOTHING_ON_CLOSE.  processWindowEvent
+       above handles this */
+    close_action = operation;
   }
 
   /**
