@@ -1,5 +1,5 @@
 /* JInternalFrame.java --
-   Copyright (C) 2002, 2004  Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -76,6 +76,8 @@ public class JInternalFrame extends JComponent implements Accessible,
   protected class AccessibleJInternalFrame extends AccessibleJComponent
     implements AccessibleValue
   {
+    private static final long serialVersionUID = 5931936924175476797L;
+
     /**
      * Creates a new AccessibleJInternalFrame object.
      */
@@ -168,6 +170,8 @@ public class JInternalFrame extends JComponent implements Accessible,
     protected class AccessibleJDesktopIcon extends AccessibleJComponent
       implements AccessibleValue
     {
+      private static final long serialVersionUID = 5035560458941637802L;
+
       /**
        * Creates a new AccessibleJDesktopIcon object.
        */
@@ -239,6 +243,8 @@ public class JInternalFrame extends JComponent implements Accessible,
       }
     }
 
+    private static final long serialVersionUID = 4672973344731387687L;
+
     /** The JInternalFrame this DesktopIcon represents. */
     JInternalFrame frame;
 
@@ -424,8 +430,19 @@ public class JInternalFrame extends JComponent implements Accessible,
   /** Whether the JInternalFrame can be maximized. */
   protected boolean maximizable;
 
-  /** Whether the JInternalFrame has rootPaneChecking enabled. */
-  protected boolean rootPaneCheckingEnabled = true;
+  /**
+   * Whether the JInternalFrame has rootPaneChecking enabled.
+   *
+   * @specnote Should be false to comply with J2SE 5.0
+   */
+  protected boolean rootPaneCheckingEnabled = false;
+
+  /**
+   * Tells us if we're in the initialization stage.
+   * If so, adds go to top-level Container, otherwise they go
+   * to the content pane for this container.
+   */
+  private boolean initStageDone = false;
 
   /** Whether the JInternalFrame is resizable. */
   protected boolean resizable;
@@ -548,12 +565,9 @@ public class JInternalFrame extends JComponent implements Accessible,
     this.maximizable = maximizable;
     this.iconable = iconifiable;
     storedBounds = new Rectangle();
-
-    setRootPaneCheckingEnabled(false);
     setRootPane(createRootPane());
-
     updateUI();
-    setRootPaneCheckingEnabled(true);
+    initStageDone = true; // Done the init stage, now adds go to content pane.
   }
 
   /**
@@ -570,10 +584,17 @@ public class JInternalFrame extends JComponent implements Accessible,
    */
   protected void addImpl(Component comp, Object constraints, int index)
   {
-    if (isRootPaneCheckingEnabled())
-      throw new Error("Do not use add() on JInternalPane directly. Use getContentPane().add() instead");
-
-    super.addImpl(comp, constraints, index);
+    // If we're adding the rootPane (initialization stages) use super.add.
+    // otherwise pass the add onto the content pane.
+    if (comp==rootPane)
+      super.addImpl(comp,constraints, index);
+    else
+      {
+        if (isRootPaneCheckingEnabled())
+          throw new Error("Do not use add() on JInternalFrame directly. Use "
+                           + "getContentPane().add() instead");
+        getContentPane().add(comp, constraints, index);
+      }
   }
 
   /**
@@ -780,9 +801,11 @@ public class JInternalFrame extends JComponent implements Accessible,
    * This method returns null because this must always be the root of a focus
    * traversal.
    *
-   * @return null.
+   * @return always null
+   *
+   * @since 1.4
    */
-  public Container getFocusCycleRootAncestor()
+  public final Container getFocusCycleRootAncestor()
   {
     // as defined.
     return null;
@@ -860,7 +883,10 @@ public class JInternalFrame extends JComponent implements Accessible,
   {
     JDesktopPane pane = getDesktopPane();
     if (pane != null)
-      return pane.getLayer(this);
+      // The cast here forces the call to the instance method getLayer()
+      // instead of the static method (this would lead to infinite
+      // recursion).
+      return pane.getLayer((Component) this);
     return -1;
   }
 
@@ -968,7 +994,7 @@ public class JInternalFrame extends JComponent implements Accessible,
    *
    * @return null.
    */
-  public String getWarningString()
+  public final String getWarningString()
   {
     // as defined.
     return null;
@@ -1019,9 +1045,11 @@ public class JInternalFrame extends JComponent implements Accessible,
   /**
    * This must always return true.
    *
-   * @return True
+   * @return always true
+   *
+   * @since 1.4
    */
-  public boolean isFocusCycleRoot()
+  public final boolean isFocusCycleRoot()
   {
     return true;
   }
@@ -1168,7 +1196,12 @@ public class JInternalFrame extends JComponent implements Accessible,
    */
   public void remove(Component comp)
   {
-    super.remove(comp);
+    // If we're removing the root pane, use super.remove.  Otherwise
+    // pass it on to the content pane instead.
+    if (comp==rootPane)
+      super.remove(comp);
+    else
+      getContentPane().remove(comp);
   }
 
   /**
@@ -1272,10 +1305,10 @@ public class JInternalFrame extends JComponent implements Accessible,
    */
   public void setDefaultCloseOperation(int operation)
   {
-    if (operation != DO_NOTHING_ON_CLOSE
-	&& operation != HIDE_ON_CLOSE
-        && operation != DISPOSE_ON_CLOSE)
-      throw new Error("Close operation must be one of DO_NOTHING_ON_CLOSE, HIDE_ON_CLOSE, or DISPOSE_ON_CLOSE");
+    /* Reference implementation allows invalid operations to be specified.
+       In that case, behaviour defaults to DO_NOTHING_ON_CLOSE.
+       processWindowEvent handles the behaviour. getDefaultCloseOperation
+       must return the invalid operator code. */
     defaultCloseOperation = operation;
   }
 
@@ -1453,9 +1486,17 @@ public class JInternalFrame extends JComponent implements Accessible,
    */
   public void setLayout(LayoutManager manager)
   {
-    if (isRootPaneCheckingEnabled())
-      throw new Error("Cannot set layout. Use getContentPane().setLayout() instead.");
-    super.setLayout(manager);
+    // Check if we're in initialization stage.  If so, call super.setLayout
+    // otherwise, valid calls go to the content pane.
+    if (initStageDone)
+      {
+        if (isRootPaneCheckingEnabled())
+          throw new Error("Cannot set layout. Use getContentPane().setLayout()"
+                           + " instead.");
+        getContentPane().setLayout(manager);
+      }
+    else
+      super.setLayout(manager);
   }
 
   /**
