@@ -89,7 +89,7 @@ import javax.accessibility.Accessible;
  * <p><b>Note:</b> the layer numbering order is the <em>reverse</em> of the
  * component indexing and position order</p>
  *
- * @author Graydon Hoare <graydon@redhat.com>
+ * @author Graydon Hoare (graydon@redhat.com)
  */
 public class JLayeredPane extends JComponent implements Accessible
 {
@@ -122,12 +122,53 @@ public class JLayeredPane extends JComponent implements Accessible
    * @return the layer the component is currently assigned to, in this container.
    * @throws IllegalArgumentException if the component is not a child of this container.
    */
-
-  protected Integer getLayer (Component c)
+  public int getLayer(Component c)
   {
     if (! componentToLayer.containsKey (c))
 	    throw new IllegalArgumentException ();
-    return (Integer) componentToLayer.get (c);
+    return ((Integer) componentToLayer.get(c)).intValue();
+  }
+
+  /**
+   * Looks up the layer of <code>comp</code> in the component's nearest
+   * JLayeredPane ancestor. If <code>comp</code> is not contained
+   * in a JLayeredPane, the value 0 (default layer) is returned.
+   *
+   * @param comp the component for which the layer is looked up
+   *
+   * @return the layer of <code>comp</code> in its nearest JLayeredPane
+   *     ancestor
+   */
+  public static int getLayer(JComponent comp)
+  {
+    JLayeredPane lp = (JLayeredPane) SwingUtilities.getAncestorOfClass
+      (JLayeredPane.class, comp);
+    if (lp == null)
+      return 0;
+    else
+      // The cast here forces the call to the instance method getLayer()
+      // instead of the static method (this would lead to infinite
+      // recursion).
+      return lp.getLayer((Component) comp);
+  }
+
+  /**
+   * Returns the first JLayeredPane that contains the Component
+   * <code>comp</code> or <code>null</code> if <code>comp</code> is
+   * not contained in a JLayeredPane.
+   *
+   * @param comp the component for which we are searching the JLayeredPane
+   *     ancestor
+   *
+   * @return the first JLayeredPane that contains the Component
+   *     <code>comp</code> or <code>null</code> if <code>comp</code> is
+   *     not contained in a JLayeredPane
+   */
+  public static JLayeredPane getLayeredPaneAbove(Component comp)
+  {
+    JLayeredPane lp = (JLayeredPane) SwingUtilities.getAncestorOfClass
+      (JLayeredPane.class, comp);
+    return lp;
   }
 
   /**
@@ -144,8 +185,7 @@ public class JLayeredPane extends JComponent implements Accessible
    * @throws IllegalArgumentException if layer does not refer to an active layer
    * in this container.
    */
-
-  protected int[] layerToRange (Integer layer)
+  private int[] layerToRange (Integer layer)
   {
     int[] ret = new int[2];
     ret[1] = getComponents ().length;
@@ -175,8 +215,7 @@ public class JLayeredPane extends JComponent implements Accessible
    * @param layer the layer number to increment.
    * @see #incrLayer()
    */
-
-  protected void incrLayer(Integer layer)
+  private void incrLayer(Integer layer)
   {
     int sz = 1;
     if (layers.containsKey (layer))
@@ -190,8 +229,7 @@ public class JLayeredPane extends JComponent implements Accessible
    * @param layer the layer number to decrement.
    * @see #decrLayer()
    */
-
-  protected void decrLayer(Integer layer)
+  private void decrLayer(Integer layer)
   {
     int sz = 0;
     if (layers.containsKey (layer))
@@ -206,7 +244,6 @@ public class JLayeredPane extends JComponent implements Accessible
    * @return the least layer number.
    * @see #lowestLayer()
    */
-
   public int highestLayer()
   {
     if (layers.size() == 0)
@@ -221,7 +258,6 @@ public class JLayeredPane extends JComponent implements Accessible
    * @return the least layer number.
    * @see #highestLayer()
    */
-    
   public int lowestLayer()
   {
     if (layers.size() == 0)
@@ -240,7 +276,6 @@ public class JLayeredPane extends JComponent implements Accessible
    * this container.
    * @see #moveToBack()
    */
-
   public void moveToFront(Component c)
   {
     setPosition (c, 0);
@@ -260,7 +295,6 @@ public class JLayeredPane extends JComponent implements Accessible
    * this container.
    * @see #moveToFront()
    */
-
   public void moveToBack(Component c)
   {
     setPosition (c, -1);
@@ -276,11 +310,10 @@ public class JLayeredPane extends JComponent implements Accessible
    * this container.
    * @see #setPosition()
    */
-    
   public int getPosition(Component c)
   {
-    Integer layer = getLayer (c);
-    int[] range = layerToRange (layer);
+    int layer = getLayer (c);
+    int[] range = layerToRange(new Integer(layer));
     int top = range[0];
     int bot = range[1];
     Component[] comps = getComponents ();
@@ -304,33 +337,32 @@ public class JLayeredPane extends JComponent implements Accessible
    * this container.
    * @see #getPosition()
    */
-
   public void setPosition(Component c, int position)
   {
-    Integer layer = getLayer (c);
-    int[] range = layerToRange (layer);
+    int layer = getLayer (c);
+    int[] range = layerToRange(new Integer(layer));
     if (range[0] == range[1])
-	    throw new IllegalArgumentException ();
+      throw new IllegalArgumentException ();
 
     int top = range[0];
     int bot = range[1];
     if (position == -1)
-	    position = (bot - top) - 1;
+      position = (bot - top) - 1;
     int targ = Math.min(top + position, bot-1);
     int curr = -1;
 
     Component[] comps = getComponents();
     for (int i = top; i < bot; ++i)
-	    {
+      {
         if (comps[i] == c)
           {
             curr = i;
             break;
           }
-	    }
+      }
     if (curr == -1)
-	    // should have found it
-	    throw new IllegalArgumentException ();
+      // should have found it
+      throw new IllegalArgumentException();
 
     super.swapComponents (curr, targ);
     revalidate();
@@ -345,7 +377,6 @@ public class JLayeredPane extends JComponent implements Accessible
    * @param layer the layer to return components from.
    * @return the components in the layer.
    */
-
   public Component[] getComponentsInLayer(int layer)
   {
     int[] range = layerToRange (getObjectForLayer (layer));
@@ -369,7 +400,6 @@ public class JLayeredPane extends JComponent implements Accessible
    * @param layer the layer count components in.
    * @return the number of components in the layer.
    */
-
   public int getComponentCountInLayer(int layer)
   {
     int[] range = layerToRange (getObjectForLayer (layer));
@@ -383,7 +413,6 @@ public class JLayeredPane extends JComponent implements Accessible
    * Return a hashtable mapping child components of this container to
    * Integer objects representing the component's layer assignments.
    */
-
   protected Hashtable getComponentToLayer()
   {
     return componentToLayer;
@@ -400,11 +429,10 @@ public class JLayeredPane extends JComponent implements Accessible
    * @throws IllegalArgumentException if the component is not a child of
    * this container.
    */
-
   public int getIndexOf(Component c) 
   {
-    Integer layer = getLayer (c);
-    int[] range = layerToRange (layer);
+    int layer = getLayer (c);
+    int[] range = layerToRange(new Integer(layer));
     Component[] comps = getComponents();
     for (int i = range[0]; i < range[1]; ++i)
 	    {
@@ -423,7 +451,6 @@ public class JLayeredPane extends JComponent implements Accessible
    * @param layer the layer number as an int.
    * @return the layer number as an Integer, possibly shared.
    */
-
   protected Integer getObjectForLayer(int layer)
   {
     switch (layer)
@@ -462,7 +489,6 @@ public class JLayeredPane extends JComponent implements Accessible
    * @param position the position in the layer at which to insert a component.
    * @return the index at which to insert the component.
    */
-    
   protected int insertIndexForLayer(int layer, int position)
   {
 
@@ -488,12 +514,11 @@ public class JLayeredPane extends JComponent implements Accessible
    *
    * @param index the index of the child component to remove.
    */
-    
   public void remove (int index)
   {
     Component c = getComponent (index);
-    Integer layer = getLayer (c);
-    decrLayer (layer);
+    int layer = getLayer (c);
+    decrLayer (new Integer(layer));
     componentToLayer.remove (c);
     super.remove (index);
     revalidate();
@@ -506,7 +531,6 @@ public class JLayeredPane extends JComponent implements Accessible
    *
    * @param comp the child to remove.
    */
-	
   public void remove (Component comp)
   {
     remove (getIndexOf (comp));
@@ -523,7 +547,6 @@ public class JLayeredPane extends JComponent implements Accessible
    * @param c the component to set the layer property for.
    * @param layer the layer number to assign to the component.
    */
-
   public void setLayer(Component c, int layer)
   {
     componentToLayer.put (c, getObjectForLayer (layer));
@@ -536,7 +559,6 @@ public class JLayeredPane extends JComponent implements Accessible
    * @param layer the layer number to assign to the component.
    * @param position the position number to assign to the component.
    */
-
   public void setLayer(Component c,
                        int layer,
                        int position)
@@ -558,7 +580,6 @@ public class JLayeredPane extends JComponent implements Accessible
    * @param layerConstraint an integer specifying the layer to add the component to.
    * @param index an ignored parameter, for compatibility.
    */
-
   protected void addImpl(Component comp, Object layerConstraint, int index) 
   {        	
     Integer layer;
@@ -578,4 +599,15 @@ public class JLayeredPane extends JComponent implements Accessible
     revalidate();
     repaint();
   }     
+
+  /**
+   * Sets the layer property for a JComponent.
+   *
+   * @param component the component for which to set the layer
+   * @param layer the layer property to set
+   */
+  public static void putLayer(JComponent component, int layer)
+  {
+    getLayeredPaneAbove(component).setLayer(component, layer);
+  }
 }
