@@ -1,5 +1,5 @@
-/* JPopupMenu.java
-   Copyright (C) 2002, 2004 Free Software Foundation, Inc.
+/* JPopupMenu.java --
+   Copyright (C) 2002, 2004, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -41,7 +41,6 @@ package javax.swing;
 import java.awt.Component;
 import java.awt.Container;
 import java.awt.Dimension;
-import java.awt.Graphics;
 import java.awt.GridBagConstraints;
 import java.awt.Insets;
 import java.awt.Panel;
@@ -53,6 +52,7 @@ import java.beans.PropertyChangeListener;
 import java.io.IOException;
 import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
+import java.util.ArrayList;
 import java.util.EventListener;
 
 import javax.accessibility.Accessible;
@@ -92,12 +92,6 @@ public class JPopupMenu extends JComponent implements Accessible, MenuElement
 {
   private static final long serialVersionUID = -8336996630009646009L;
 
-  /** name for the UI delegate for this menuItem. */
-  private static final String uiClassID = "PopupMenuUI";
-
-  /** Fire a PropertyChangeEvent when the "borderPainted" property changes. */
-  public static final String LABEL_CHANGED_PROPERTY = "label";
-
   /* indicates if popup's menu border should be painted*/
   private boolean borderPainted = true;
 
@@ -142,19 +136,12 @@ public class JPopupMenu extends JComponent implements Accessible, MenuElement
   /* Field indicating if popup menu is visible or not */
   private boolean visible = false;
   
-  /* Bound Property indicating visibility of the popup menu*/
-  public static final String VISIBLE_CHANGED_PROPERTY = "visible";
-
   /**
    * Creates a new JPopupMenu object.
    */
   public JPopupMenu()
   {
-    updateUI();
-
-    lightWeightPopupEnabled = DefaultLightWeightPopupEnabled;
-    selectionModel = new DefaultSingleSelectionModel();
-    super.setVisible(false);
+    this(null);
   }
 
   /**
@@ -164,7 +151,11 @@ public class JPopupMenu extends JComponent implements Accessible, MenuElement
    */
   public JPopupMenu(String label)
   {
+    lightWeightPopupEnabled = getDefaultLightWeightPopupEnabled();
     setLabel(label);
+    setSelectionModel(new DefaultSingleSelectionModel());
+    super.setVisible(false);
+    updateUI();
   }
 
   private void readObject(ObjectInputStream stream)
@@ -270,9 +261,6 @@ public class JPopupMenu extends JComponent implements Accessible, MenuElement
     constraints.weightx = 100.0;
     constraints.weighty = 100.0;
 
-    if (index == -1)
-      index = getComponents().length;
-
     constraints.gridy = index;
     super.add(component, constraints, index);
 
@@ -290,18 +278,6 @@ public class JPopupMenu extends JComponent implements Accessible, MenuElement
       }
   }
 
-  /**
-   * Paints popup menu's border if borderPainted is true
-   *
-   * @param graphics graphics context used to paint this popup's menu border.
-   */
-  protected void borderPainted(Graphics graphics)
-  {
-    if (borderPainted)
-      getBorder().paintBorder(this, graphics, 0, 0, getSize(null).width,
-                              getSize(null).height);
-  }
-
   /**
    * Returns flag indicating if newly created JPopupMenu will use
    * heavyweight or lightweight container to display its menu items
@@ -449,7 +425,7 @@ public class JPopupMenu extends JComponent implements Accessible, MenuElement
   /**
    * Sets label for this popup menu. This method fires PropertyChangeEvent
    * when the label property is changed. Please note that most
-   * of the Look & Feel will ignore this property.
+   * of the Look &amp; Feel will ignore this property.
    *
    * @param label label for this popup menu
    */
@@ -459,7 +435,7 @@ public class JPopupMenu extends JComponent implements Accessible, MenuElement
       {
 	String oldLabel = this.label;
 	this.label = label;
-	firePropertyChange(LABEL_CHANGED_PROPERTY, oldLabel, label);
+	firePropertyChange("label", oldLabel, label);
       }
   }
 
@@ -571,11 +547,14 @@ public class JPopupMenu extends JComponent implements Accessible, MenuElement
    */
   public void setVisible(boolean visible)
   {
+    if (visible == isVisible())
+      return;
+
     boolean old = isVisible();
     this.visible = visible;
     if (old != isVisible())
       {
-	firePropertyChange(VISIBLE_CHANGED_PROPERTY, old, (boolean) isVisible());
+	firePropertyChange("visible", old, isVisible());
 	if (visible)
 	  {
 	    firePopupMenuWillBecomeVisible();
@@ -788,7 +767,17 @@ public class JPopupMenu extends JComponent implements Accessible, MenuElement
    */
   protected String paramString()
   {
-    return "JPopupMenu";
+    StringBuffer sb = new StringBuffer();
+    sb.append(super.paramString());
+    sb.append(",label=");
+    if (getLabel() != null)
+      sb.append(getLabel());
+    sb.append(",lightWeightPopupEnabled=").append(isLightWeightPopupEnabled());
+    sb.append(",margin=");
+    if (getMargin() != null)
+      sb.append(margin);
+    sb.append(",paintBorder=").append(isBorderPainted());
+    return sb.toString();
   }
 
   /**
@@ -836,19 +825,22 @@ public class JPopupMenu extends JComponent implements Accessible, MenuElement
   }
 
   /**
-   * Return subcomonents of this popup menu.
+   * Return subcomonents of this popup menu. This method returns only
+   * components that implement the <code>MenuElement</code> interface.
    *
-   * @return Array containing menuItem's of belonging to this popup menu.
+   * @return array of menu items belonging to this popup menu
    */
   public MenuElement[] getSubElements()
   {
     Component[] items = getComponents();
-    MenuElement[] subElements = new MenuElement[items.length];
+    ArrayList subElements = new ArrayList();
 
     for (int i = 0; i < items.length; i++)
-      subElements[i] = (MenuElement) items[i];
+      if (items[i] instanceof MenuElement)
+	subElements.add(items[i]);
 
-    return subElements;
+    return (MenuElement[])
+      subElements.toArray(new MenuElement[subElements.size()]);
   }
 
   /**
@@ -1035,14 +1027,6 @@ public class JPopupMenu extends JComponent implements Accessible, MenuElement
       this.setBounds(x, y, width, height);
       this.show();
     }
-
-    /**
-     * Hides JWindow with menu item's from the screen.
-     */
-    public void hide()
-    {
-      super.hide();
-    }
   }
 
   /**
