@@ -182,17 +182,6 @@ public class JSeparator extends JComponent implements SwingConstants,
     return "JSeparator";
   }
 
-  /**
-   * This method overrides the isFocusTraversable method from
-   * Component to false. JSeparator cannot be focused on.
-   *
-   * @return boolean False.
-   */
-  public boolean isFocusTraversable()
-  {
-    return false;
-  }
-
   /**
    * getAccessibleContext
    *
