@@ -1,5 +1,5 @@
 /* JTable.java -- 
-   Copyright (C) 2002, 2004  Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -41,7 +41,11 @@ package javax.swing;
 import java.awt.Color;
 import java.awt.Component;
 import java.awt.Dimension;
+import java.awt.Point;
 import java.awt.Rectangle;
+import java.text.DateFormat;
+import java.text.NumberFormat;
+import java.util.Date;
 import java.util.Hashtable;
 import java.util.Vector;
 
@@ -70,6 +74,212 @@ public class JTable extends JComponent
   implements TableModelListener, Scrollable, TableColumnModelListener,
              ListSelectionListener, CellEditorListener, Accessible
 {
+
+  /**
+   * A cell renderer for boolean values.
+   */
+  private class BooleanCellRenderer
+    extends DefaultTableCellRenderer
+  {
+
+    /**
+     * The CheckBox that is used for rendering.
+     */
+    private JCheckBox checkBox = new JCheckBox();
+
+    /**
+     * Returns the component that is used for rendering the value.
+     *
+     * @param table the JTable
+     * @param value the value of the object
+     * @param isSelected is the cell selected?
+     * @param hasFocus has the cell the focus?
+     * @param row the row to render
+     * @param column the cell to render
+     * 
+     * @return this component (the default table cell renderer)
+     */
+    public Component getTableCellRendererComponent(JTable table, Object value,
+                                                   boolean isSelected,
+                                                   boolean hasFocus, int row,
+                                                   int column)
+    {
+      Boolean boolValue = (Boolean) value;
+      checkBox.setSelected(boolValue.booleanValue());
+      return checkBox;
+    }
+  }
+
+  /**
+   * A cell renderer for Date values.
+   */
+  private class DateCellRenderer
+    extends DefaultTableCellRenderer
+  {
+    /**
+     * Returns the component that is used for rendering the value.
+     *
+     * @param table the JTable
+     * @param value the value of the object
+     * @param isSelected is the cell selected?
+     * @param hasFocus has the cell the focus?
+     * @param row the row to render
+     * @param column the cell to render
+     * 
+     * @return this component (the default table cell renderer)
+     */
+    public Component getTableCellRendererComponent(JTable table, Object value,
+                                                   boolean isSelected,
+                                                   boolean hasFocus, int row,
+                                                   int column)
+    {
+      super.getTableCellRendererComponent(table, value, isSelected, hasFocus,
+                                          row, column);
+      if (value instanceof Date)
+        {
+          Date dateValue = (Date) value;
+          DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT);
+          setText(df.format(dateValue));
+        }
+      return this;
+    }
+  }
+
+  /**
+   * A cell renderer for Double values.
+   */
+  private class DoubleCellRenderer
+    extends DefaultTableCellRenderer
+  {
+    /**
+     * Creates a new instance of NumberCellRenderer.
+     */
+    public DoubleCellRenderer()
+    {
+      setHorizontalAlignment(JLabel.RIGHT);
+    }
+
+    /**
+     * Returns the component that is used for rendering the value.
+     *
+     * @param table the JTable
+     * @param value the value of the object
+     * @param isSelected is the cell selected?
+     * @param hasFocus has the cell the focus?
+     * @param row the row to render
+     * @param column the cell to render
+     * 
+     * @return this component (the default table cell renderer)
+     */
+    public Component getTableCellRendererComponent(JTable table, Object value,
+                                                   boolean isSelected,
+                                                   boolean hasFocus, int row,
+                                                   int column)
+    {
+      super.getTableCellRendererComponent(table, value, isSelected, hasFocus,
+                                          row, column);
+      if (value instanceof Double)
+        {
+          Double doubleValue = (Double) value;
+          NumberFormat nf = NumberFormat.getInstance();
+          setText(nf.format(doubleValue.doubleValue()));
+        }
+      return this;
+    }
+  }
+
+  /**
+   * A cell renderer for Float values.
+   */
+  private class FloatCellRenderer
+    extends DefaultTableCellRenderer
+  {
+    /**
+     * Creates a new instance of NumberCellRenderer.
+     */
+    public FloatCellRenderer()
+    {
+      setHorizontalAlignment(JLabel.RIGHT);
+    }
+
+    /**
+     * Returns the component that is used for rendering the value.
+     *
+     * @param table the JTable
+     * @param value the value of the object
+     * @param isSelected is the cell selected?
+     * @param hasFocus has the cell the focus?
+     * @param row the row to render
+     * @param column the cell to render
+     * 
+     * @return this component (the default table cell renderer)
+     */
+    public Component getTableCellRendererComponent(JTable table, Object value,
+                                                   boolean isSelected,
+                                                   boolean hasFocus, int row,
+                                                   int column)
+    {
+      super.getTableCellRendererComponent(table, value, isSelected, hasFocus,
+                                          row, column);
+      if (value instanceof Float)
+        {
+          Float floatValue = (Float) value;
+          NumberFormat nf = NumberFormat.getInstance();
+          setText(nf.format(floatValue.floatValue()));
+        }
+      return this;
+    }
+  }
+
+  /**
+   * A cell renderer for Number values.
+   */
+  private class NumberCellRenderer
+    extends DefaultTableCellRenderer
+  {
+    /**
+     * Creates a new instance of NumberCellRenderer.
+     */
+    public NumberCellRenderer()
+    {
+      setHorizontalAlignment(JLabel.RIGHT);
+    }
+  }
+
+  /**
+   * A cell renderer for Icon values.
+   */
+  private class IconCellRenderer
+    extends DefaultTableCellRenderer
+  {
+    /**
+     * Returns the component that is used for rendering the value.
+     *
+     * @param table the JTable
+     * @param value the value of the object
+     * @param isSelected is the cell selected?
+     * @param hasFocus has the cell the focus?
+     * @param row the row to render
+     * @param column the cell to render
+     * 
+     * @return this component (the default table cell renderer)
+     */
+    public Component getTableCellRendererComponent(JTable table, Object value,
+                                                   boolean isSelected,
+                                                   boolean hasFocus, int row,
+                                                   int column)
+    {
+      super.getTableCellRendererComponent(table, value, isSelected, hasFocus,
+                                          row, column);
+      if (value instanceof Icon)
+        {
+          Icon iconValue = (Icon) value;
+          setIcon(iconValue);
+        }
+      return this;
+    }
+  }
+
   private static final long serialVersionUID = 3876025080382781659L;
 
 
@@ -278,7 +488,7 @@ public class JTable extends JComponent
    * @see #setDragEnabled()
    * @see #getDragEnabled()
    */
-  protected boolean dragEnabled;
+  private boolean dragEnabled;
 
   /**
    * The color to paint the grid lines of the table, when either {@link
@@ -296,7 +506,7 @@ public class JTable extends JComponent
    * @see #setPreferredScrollableViewportSize()
    * @see #getPreferredScrollableViewportSize()
    */
-  protected Dimension preferredScrollableViewportSize;
+  protected Dimension preferredViewportSize;
 
   /**
    * The color to paint the background of selected cells. Fires a property
@@ -406,7 +616,7 @@ public class JTable extends JComponent
    */
   public JTable (TableModel dm, TableColumnModel cm, ListSelectionModel sm)
   {
-    this.dataModel = dm == null ? createDefaultDataModel() : dm;
+    setModel(dm == null ? createDefaultDataModel() : dm);
     setSelectionModel(sm == null ? createDefaultSelectionModel() : sm);
 
     this.columnModel = cm;
@@ -416,6 +626,7 @@ public class JTable extends JComponent
 
   protected void initializeLocalVars()
   {
+    setTableHeader(createDefaultTableHeader());
     this.autoCreateColumnsFromModel = false;
     if (columnModel == null)
       {
@@ -437,13 +648,12 @@ public class JTable extends JComponent
     // this.accessibleContext = new AccessibleJTable();
     this.cellEditor = null;
     this.dragEnabled = false;
-    this.preferredScrollableViewportSize = new Dimension(450,400);
+    this.preferredViewportSize = new Dimension(450,400);
     this.showHorizontalLines = true;
     this.showVerticalLines = true;
     this.editingColumn = -1;
     this.editingRow = -1;
     setIntercellSpacing(new Dimension(1,1));
-    setTableHeader(createDefaultTableHeader());
   }
 
   /**
@@ -475,7 +685,12 @@ public class JTable extends JComponent
 
   protected void createDefaultRenderers()
   {
-    //FIXME: Create the renderer object.
+    setDefaultRenderer(Boolean.class, new BooleanCellRenderer());
+    setDefaultRenderer(Number.class, new NumberCellRenderer());
+    setDefaultRenderer(Double.class, new DoubleCellRenderer());
+    setDefaultRenderer(Double.class, new FloatCellRenderer());
+    setDefaultRenderer(Date.class, new DateCellRenderer());
+    setDefaultRenderer(Icon.class, new IconCellRenderer());
   }
   
   /**
@@ -563,6 +778,13 @@ public class JTable extends JComponent
 
   public void tableChanged (TableModelEvent event)
   {
+    // update the column model from the table model if the structure has
+    // changed and the flag autoCreateColumnsFromModel is set
+    if ((event.getFirstRow() ==TableModelEvent.HEADER_ROW)
+        && autoCreateColumnsFromModel)
+
+        createColumnsFromModel();
+
     repaint();
   }
 
@@ -571,6 +793,58 @@ public class JTable extends JComponent
     repaint();
   }
 
+ /**
+   * Returns index of the column that contains specified point 
+   * or -1 if this table doesn't contain this point.
+   *
+   * @param point point to identify the column
+   * @return index of the column that contains specified point or 
+   * -1 if this table doesn't contain this point.
+   */
+  public int columnAtPoint(Point point)
+  {
+    int x0 = getLocation().x;
+    int ncols = getColumnCount();
+    Dimension gap = getIntercellSpacing();
+    TableColumnModel cols = getColumnModel();
+    int x = point.x;
+    
+    for (int i = 0; i < ncols; ++i)
+      {
+        int width = cols.getColumn(i).getWidth() + (gap == null ? 0 : gap.width);
+        if (0 <= x && x < width)
+          return i;
+        x -= width;  
+      }
+    
+    return -1;
+  }
+
+  /**
+   * Returns index of the row that contains specified point or 
+   * -1 if this table doesn't contain this point.
+   *
+   * @param point point to identify the row
+   * @return index of the row that contains specified point or 
+   * -1 if this table doesn't contain this point.
+   */
+  public int rowAtPoint(Point point)
+  {
+    int y0 = getLocation().y;
+    int nrows = getRowCount();
+    Dimension gap = getIntercellSpacing();
+    int height = getRowHeight() + (gap == null ? 0 : gap.height);
+    int y = point.y;
+    
+    for (int i = 0; i < nrows; ++i)
+      {
+        if (0 <= y && y < height)
+          return i;
+        y -= height;
+      }
+      
+    return -1;
+  }
 
   /** 
    * Calculate the visible rectangle for a particular row and column. The
@@ -921,11 +1195,11 @@ public class JTable extends JComponent
             break;
             
           case ListSelectionModel.SINGLE_INTERVAL_SELECTION:
-            sum = hi - lo;
+            sum = hi - lo + 1;
             break;
             
           case ListSelectionModel.MULTIPLE_INTERVAL_SELECTION:        
-            for (int i = lo; i < hi; ++i)
+            for (int i = lo; i <= hi; ++i)
               if (lsm.isSelectedIndex(i))        
                 ++sum;
             break;
@@ -952,12 +1226,12 @@ public class JTable extends JComponent
             break;      
       
           case ListSelectionModel.SINGLE_INTERVAL_SELECTION:            
-            for (int i = lo; i < hi; ++i)
+            for (int i = lo; i <= hi; ++i)
               ret[j++] = i;
             break;
             
           case ListSelectionModel.MULTIPLE_INTERVAL_SELECTION:        
-            for (int i = lo; i < hi; ++i)
+            for (int i = lo; i <= hi; ++i)
               if (lsm.isSelectedIndex(i))        
                 ret[j++] = i;
             break;
@@ -1071,13 +1345,13 @@ public class JTable extends JComponent
   }
 
   /**
-   * Get the value of the {@link #preferredScrollableViewportSize} property.
+   * Get the value of the {@link #preferredViewportSize} property.
    *
    * @return The current value of the property
    */
   public Dimension getPreferredScrollableViewportSize()
   {
-    return preferredScrollableViewportSize;
+    return preferredViewportSize;
   }
 
   /**
@@ -1290,6 +1564,12 @@ public class JTable extends JComponent
         for (int i = 0; i < ncols; ++i)
           columnModel.getColumn(i).setHeaderValue(dataModel.getColumnName(i));
       }
+
+    // according to Sun's spec we also have to set the tableHeader's
+    // column model here
+    if (tableHeader != null)
+      tableHeader.setColumnModel(c);
+
     revalidate();
     repaint();
   }
@@ -1328,13 +1608,16 @@ public class JTable extends JComponent
 
   /**
    * Set the value of the {@link #selectionMode} property by
-   * delegation to the {@link #selectionModel} field.
+   * delegation to the {@link #selectionModel} field. The same selection
+   * mode is set for row and column selection models.
    *
    * @param s The new value of the property
    */ 
   public void setSelectionMode(int s)
-  {
-    selectionModel.setSelectionMode(s);
+  { 
+    selectionModel.setSelectionMode(s);    
+    columnModel.getSelectionModel().setSelectionMode(s);
+    
     repaint();
   }
 
@@ -1391,13 +1674,13 @@ public class JTable extends JComponent
   }
 
   /**
-   * Set the value of the {@link #preferredScrollableViewportSize} property.
+   * Set the value of the {@link #preferredViewportSize} property.
    *
-   * @param p The new value of the preferredScrollableViewportSize property
+   * @param p The new value of the preferredViewportSize property
    */ 
   public void setPreferredScrollableViewportSize(Dimension p)
   {
-    preferredScrollableViewportSize = p;
+    preferredViewportSize = p;
     revalidate();
     repaint();
   }
@@ -1524,44 +1807,22 @@ public class JTable extends JComponent
 
 
   /**
-   * Sun javadocs describe an unusual implementation of
-   * <code>doLayout</code> which involves some private interfaces. We try
-   * to implement the same algorithm as is documented, but using the
-   * columnModel directly. We still use a private helper method, but it has
-   * a simpler signature.
+   * This distributes the superfluous width in a table evenly on its columns.
+   *
+   * The implementation used here is different to that one described in
+   * the JavaDocs. It is much simpler, and seems to work very well.
+   *
+   * TODO: correctly implement the algorithm described in the JavaDoc
    */
-
   private void distributeSpill(TableColumn[] cols, int spill)
   {
-    int MIN = 0;
-    int MAX = 0;
-    int PREF = 0;
-
-    int[] min = new int[cols.length];
-    int[] max = new int[cols.length];
-    int[] pref = new int[cols.length];
-
-    for (int i = 0; i < cols.length; ++i)
+    int average = spill / cols.length;
+    for (int i = 0; i < cols.length; i++)
       {
-        pref[i] = cols[i].getPreferredWidth();
-        min[i] = cols[i].getMinWidth();
-        max[i] = cols[i].getMaxWidth();
-        PREF += pref[i];
-        MIN += min[i];
-        MAX += max[i];
+        cols[i].setWidth(cols[i].getWidth() + average);
       }
-
-    for (int i = 0; i < cols.length; ++i)
-      {
-        int adj = 0;
-        if (spill > 0)          
-          adj = (spill * (pref[i] - min[i])) / (PREF - MIN);
-        else
-          adj = (spill * (max[i] - pref[i])) / (MAX - PREF);
-        cols[i].setWidth(pref[i] + adj);        
-      }    
   }
-  
+
   public void doLayout()
   {
     TableColumn resizingColumn = null;
@@ -1587,7 +1848,7 @@ public class JTable extends JComponent
           rCol = i;
       }
 
-    int spill = prefSum - getWidth();
+    int spill = getWidth() - prefSum;
 
     if (resizingColumn != null)
       {
@@ -1803,4 +2064,19 @@ public class JTable extends JComponent
     setColumnSelectionInterval(0, getColumnCount() - 1);
     setRowSelectionInterval(0, getRowCount() - 1);
   }
+
+  public Object getValueAt(int row, int column)
+  {
+    return dataModel.getValueAt(row, convertColumnIndexToModel(column));
+  }
+
+  public void setValueAt(Object value, int row, int column)
+  {
+    dataModel.setValueAt(value, row, convertColumnIndexToModel(column));
+  }
+
+  public TableColumn getColumn(Object identifier)
+  {
+    return columnModel.getColumn(columnModel.getColumnIndex(identifier));
+  }
 }
