@@ -1,5 +1,5 @@
 /* JTextField.java --
-   Copyright (C) 2002, 2004  Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -47,9 +47,12 @@ import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 
 import javax.accessibility.AccessibleStateSet;
+import javax.swing.text.AttributeSet;
+import javax.swing.text.BadLocationException;
 import javax.swing.text.Document;
 import javax.swing.text.JTextComponent;
 import javax.swing.text.PlainDocument;
+import javax.swing.text.TextAction;
 
 public class JTextField extends JTextComponent
   implements SwingConstants
@@ -80,12 +83,29 @@ public class JTextField extends JTextComponent
 
   private static final long serialVersionUID = 353853209832607592L;
 
+  private static final Action[] actions;
+
+  /**
+   * Name of the action that gets sent when the content of the text field
+   * gets accepted.
+   */
   public static final String notifyAction = "notify-field-accept";
   
+  static
+  {
+    actions = new Action[1];
+    actions[0] = new TextAction(notifyAction)
+      {
+	public void actionPerformed(ActionEvent event)
+	{
+	  JTextField textField = (JTextField) event.getSource();
+	  textField.fireActionPerformed();
+	}
+      };
+  }
+  
   private int columns;
-
   private int align;
-
   private int scrollOffset;
 
   /** @since 1.3 */
@@ -159,6 +179,9 @@ public class JTextField extends JTextComponent
 
     if (text != null)
       setText(text);
+
+    // default value for alignment
+    align = LEADING;
   }
 
   /**
@@ -169,7 +192,15 @@ public class JTextField extends JTextComponent
    */
   protected Document createDefaultModel()
   {
-    return new PlainDocument();
+    // subclassed to swallow newlines
+    return new PlainDocument() {
+        public void insertString(int offset, String str, AttributeSet a)
+          throws BadLocationException
+        {
+          if (str.indexOf('\n') == -1)
+            super.insertString(offset, str, a);
+        }
+      };
   }
 
   /**
@@ -300,9 +331,15 @@ public class JTextField extends JTextComponent
     scrollOffset = offset;
   }
 
+  public Action[] getActions()
+  {
+    return TextAction.augmentList(super.getActions(), actions);
+  }
+
   public void postActionEvent()
   {
-    ActionEvent event = new ActionEvent(this, 0, actionCommand);
+    String command = actionCommand != null ? actionCommand : getText();
+    ActionEvent event = new ActionEvent(this, 0, command);
     ActionListener[] listeners = getActionListeners();
 
     for (int index = 0; index < listeners.length; ++index)
@@ -346,20 +383,12 @@ public class JTextField extends JTextComponent
     firePropertyChange("horizontalAlignment", oldAction, newAction);
   }
 
-  /**
-   * @since 1.3
-   */
-  public String getActionCommand()
-  {
-    return actionCommand;
-  }
-
   /**
    * @since 1.3
    */
   public void setActionCommand(String command)
   {
-    this.actionCommand = command;
+    actionCommand = command;
   }
 
   /**
