@@ -63,9 +63,21 @@ public class JWindow extends Window implements Accessible, RootPaneContainer
   private static final long serialVersionUID = 5420698392125238833L;
   
   protected JRootPane rootPane;
-  protected boolean rootPaneCheckingEnabled;
+
+  /**
+   * @specnote rootPaneCheckingEnabled is false to comply with J2SE 5.0
+   */
+  protected boolean rootPaneCheckingEnabled = false;
+
   protected AccessibleContext accessibleContext;
 
+  /**
+   * Tells us if we're in the initialization stage.
+   * If so, adds go to top-level Container, otherwise they go
+   * to the content pane for this container.
+   */
+  private boolean initStageDone = false;
+
   public JWindow()
   {
     super(SwingUtilities.getOwnerFrame());
@@ -100,6 +112,8 @@ public class JWindow extends Window implements Accessible, RootPaneContainer
   {
     super.setLayout(new BorderLayout(1, 1));
     getRootPane(); // will do set/create
+    // Now we're done init stage, adds and layouts go to content pane.
+    initStageDone = true;
   }
 
   public Dimension getPreferredSize()
@@ -109,7 +123,17 @@ public class JWindow extends Window implements Accessible, RootPaneContainer
 
   public void setLayout(LayoutManager manager)
   {
-    super.setLayout(manager);
+    // Check if we're in initialization stage.  If so, call super.setLayout
+    // otherwise, valid calls go to the content pane.
+    if (initStageDone)
+      {
+        if (isRootPaneCheckingEnabled())
+          throw new Error("Cannot set layout. Use getContentPane().setLayout()"
+                           + " instead.");
+        getContentPane().setLayout(manager);
+      }
+    else
+      super.setLayout(manager);
   }
 
   public void setLayeredPane(JLayeredPane layeredPane)
@@ -163,14 +187,30 @@ public class JWindow extends Window implements Accessible, RootPaneContainer
     getRootPane().setGlassPane(glassPane);
   }
 
+
   protected void addImpl(Component comp, Object constraints, int index)
   {
-    super.addImpl(comp, constraints, index);
+    // If we're adding the rootPane (initialization stages) use super.add.
+    // otherwise pass the add onto the content pane.
+    if (comp == rootPane)
+      super.addImpl(comp, constraints, index);
+    else
+      {
+        if (isRootPaneCheckingEnabled())
+          throw new Error("Do not use add() on JWindow directly. Use "
+                          + "getContentPane().add() instead");
+        getContentPane().add(comp, constraints, index);
+      }
   }
 
   public void remove(Component comp)
   {
-    getContentPane().remove(comp);
+    // If we're removing the root pane, use super.remove.  Otherwise
+    // pass it on to the content pane instead.
+    if (comp == rootPane)
+      super.remove(rootPane);
+    else
+      getContentPane().remove(comp);
   }
 
   protected boolean isRootPaneCheckingEnabled()
