@@ -1,5 +1,5 @@
 /* DefaultRGHChooserPanel.java --
-   Copyright (C) 2004 Free Software Foundation, Inc.
+   Copyright (C) 2004, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -57,7 +57,7 @@ import javax.swing.event.ChangeListener;
  * This is the default RGB panel for the JColorChooser. The color is selected
  * using three sliders that represent the RGB values.
  */
-public class DefaultRGBChooserPanel extends AbstractColorChooserPanel
+class DefaultRGBChooserPanel extends AbstractColorChooserPanel
 {
   /**
    * This class handles the slider value changes for all three sliders.
@@ -109,17 +109,20 @@ public class DefaultRGBChooserPanel extends AbstractColorChooserPanel
     }
   }
 
-  /** Whether the color change was initiated by the spinners. */
-  private transient boolean spinnerChange = false;
+  /** Whether the color change was initiated by the spinners.
+   * This is package-private to avoid an accessor method.  */
+  transient boolean spinnerChange = false;
 
-  /** Whether the color change was initiated by the sliders. */
-  private transient boolean sliderChange = false;
+  /** Whether the color change was initiated by the sliders.
+   * This is package-private to avoid an accessor method.  */
+  transient boolean sliderChange = false;
 
   /**
    * Whether the change was forced by the chooser (meaning the color has
    * already been changed).
+   * This is package-private to avoid an accessor method.
    */
-  private transient boolean updateChange = false;
+  transient boolean updateChange = false;
 
   /** The ChangeListener for the sliders. */
   private transient ChangeListener colorChanger;
@@ -127,14 +130,17 @@ public class DefaultRGBChooserPanel extends AbstractColorChooserPanel
   /** The ChangeListener for the spinners. */
   private transient ChangeListener spinnerHandler;
 
-  /** The slider that handles the red values. */
-  private transient JSlider R;
+  /** The slider that handles the red values.
+   * This is package-private to avoid an accessor method.  */
+  transient JSlider R;
 
-  /** The slider that handles the green values. */
-  private transient JSlider G;
+  /** The slider that handles the green values.
+   * This is package-private to avoid an accessor method.  */
+  transient JSlider G;
 
-  /** The slider that handles the blue values. */
-  private transient JSlider B;
+  /** The slider that handles the blue values.
+   * This is package-private to avoid an accessor method.  */
+  transient JSlider B;
 
   /** The label for the red slider. */
   private transient JLabel RLabel;
@@ -145,14 +151,17 @@ public class DefaultRGBChooserPanel extends AbstractColorChooserPanel
   /** The label for the blue slider. */
   private transient JLabel BLabel;
 
-  /** The spinner that handles the red values. */
-  private transient JSpinner RSpinner;
+  /** The spinner that handles the red values.
+   * This is package-private to avoid an accessor method.  */
+  transient JSpinner RSpinner;
 
-  /** The spinner that handles the green values. */
-  private transient JSpinner GSpinner;
+  /** The spinner that handles the green values.
+   * This is package-private to avoid an accessor method.  */
+  transient JSpinner GSpinner;
 
-  /** The spinner that handles the blue values. */
-  private transient JSpinner BSpinner;
+  /** The spinner that handles the blue values.
+   * This is package-private to avoid an accessor method.  */
+  transient JSpinner BSpinner;
 
   /**
    * Creates a new DefaultRGBChooserPanel object.
