@@ -176,12 +176,12 @@ public class BasicLabelUI extends LabelUI implements PropertyChangeListener
     if (icon != null)
       icon.paintIcon(b, g, ir.x, ir.y);
     if (text != null && ! text.equals(""))
-    {
-      if (b.isEnabled())
-        paintEnabledText(b, g, text, tr.x, tr.y + fm.getAscent());
-      else
-        paintDisabledText(b, g, text, tr.x, tr.y + fm.getAscent());
-    }
+      {
+	if (b.isEnabled())
+	  paintEnabledText(b, g, text, tr.x, tr.y + fm.getAscent());
+	else
+	  paintDisabledText(b, g, text, tr.x, tr.y + fm.getAscent());
+      }
     g.setFont(saved_font);
   }
 
@@ -350,8 +350,6 @@ public class BasicLabelUI extends LabelUI implements PropertyChangeListener
     c.setForeground(defaults.getColor("Label.foreground"));
     c.setBackground(defaults.getColor("Label.background"));
     c.setFont(defaults.getFont("Label.font"));
-    c.setBorder(defaults.getBorder("Label.border"));
-    c.setOpaque(true);
     //XXX: There are properties we don't use called disabledForeground
     //and disabledShadow.
   }
@@ -367,7 +365,6 @@ public class BasicLabelUI extends LabelUI implements PropertyChangeListener
     c.setForeground(null);
     c.setBackground(null);
     c.setFont(null);
-    c.setBorder(null);
   }
 
   /**
