@@ -49,6 +49,8 @@ import java.awt.event.ComponentEvent;
 import java.awt.event.ComponentListener;
 import java.awt.event.FocusEvent;
 import java.awt.event.FocusListener;
+import java.awt.event.KeyAdapter;
+import java.awt.event.KeyEvent;
 import java.awt.event.MouseEvent;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
@@ -183,6 +185,50 @@ public class BasicListUI extends ListUI
     }
   }
 
+  /**
+   * A helper class which listens for {@link KeyEvents}s 
+   * from the {@link JList}.
+   */
+  private class KeyHandler extends KeyAdapter
+  {
+    public KeyHandler()
+    {
+    }
+    
+    public void keyPressed( KeyEvent evt ) 
+    {
+      if (evt.getKeyCode() == KeyEvent.VK_DOWN)
+        {
+          int lead = BasicListUI.this.list.getLeadSelectionIndex();
+          int max = BasicListUI.this.list.getModel().getSize() - 1;
+          if (!evt.isShiftDown())
+            {
+              BasicListUI.this.list.clearSelection();
+              BasicListUI.this.list.setSelectedIndex(Math.min(lead+1,max));
+            }
+          else 
+            {
+              BasicListUI.this.list.getSelectionModel().
+                setLeadSelectionIndex(Math.min(lead+1,max));
+            }
+        }
+      else if (evt.getKeyCode() == KeyEvent.VK_UP)
+        {
+          int lead = BasicListUI.this.list.getLeadSelectionIndex();
+          if (!evt.isShiftDown())
+            {
+              BasicListUI.this.list.clearSelection();
+              BasicListUI.this.list.setSelectedIndex(Math.max(lead-1,0));
+            }
+          else
+            {
+              BasicListUI.this.list.getSelectionModel().
+                setLeadSelectionIndex(Math.max(lead-1,0));
+            }
+        }
+    }
+  }
+  
   /**
    * A helper class which listens for {@link MouseEvent}s 
    * from the {@link JList}.
@@ -197,6 +243,46 @@ public class BasicListUI extends ListUI
      */
     public void mouseClicked(MouseEvent event)
     {
+      Point click = event.getPoint();
+      int index = BasicListUI.this.locationToIndex(list, click);
+      if (index == -1)
+        return;
+      if (event.isControlDown())
+        {
+          if (BasicListUI.this.list.getSelectionMode() == 
+              ListSelectionModel.SINGLE_SELECTION)
+            BasicListUI.this.list.setSelectedIndex(index);
+          else if (BasicListUI.this.list.isSelectedIndex(index))
+            BasicListUI.this.list.removeSelectionInterval(index,index);
+          else
+            BasicListUI.this.list.addSelectionInterval(index,index);
+        }
+      else if (event.isShiftDown())
+        {
+          if (BasicListUI.this.list.getSelectionMode() == 
+              ListSelectionModel.SINGLE_SELECTION)
+            BasicListUI.this.list.setSelectedIndex(index);
+          else if (BasicListUI.this.list.getSelectionMode() == 
+                   ListSelectionModel.SINGLE_INTERVAL_SELECTION)
+            // COMPAT: the IBM VM is compatible with the following line of code.
+            // However, compliance with Sun's VM would correspond to replacing 
+            // getAnchorSelectionIndex() with getLeadSelectionIndex().This is 
+            // both unnatural and contradictory to the way they handle other 
+            // similar UI interactions.
+            BasicListUI.this.list.setSelectionInterval
+              (BasicListUI.this.list.getAnchorSelectionIndex(), index);
+          else
+            // COMPAT: both Sun and IBM are compatible instead with:
+            // BasicListUI.this.list.setSelectionInterval
+            //     (BasicListUI.this.list.getLeadSelectionIndex(),index);
+            // Note that for IBM this is contradictory to what they did in 
+            // the above situation for SINGLE_INTERVAL_SELECTION.  
+            // The most natural thing to do is the following:
+            BasicListUI.this.list.getSelectionModel().
+              setLeadSelectionIndex(index);
+        }
+      else
+        BasicListUI.this.list.setSelectedIndex(index);
     }
 
     /**
@@ -207,12 +293,6 @@ public class BasicListUI extends ListUI
      */
     public void mousePressed(MouseEvent event)
     {
-      Point click = event.getPoint();
-      int index = BasicListUI.this.locationToIndex(list, click);
-      if (index == -1)
-        return;
-
-      BasicListUI.this.list.setSelectedIndex(index);
     }
 
     /**
@@ -304,19 +384,23 @@ public class BasicListUI extends ListUI
   }
 
   /** The current focus listener. */
-  protected FocusHandler focusListener;
+  protected FocusListener focusListener;
 
   /** The data listener listening to the model. */
-  protected ListDataHandler listDataListener;
+  protected ListDataListener listDataListener;
 
   /** The selection listener listening to the selection model. */
-  protected ListSelectionHandler listSelectionListener;
+  protected ListSelectionListener listSelectionListener;
 
   /** The mouse listener listening to the list. */
-  protected MouseInputHandler mouseInputListener;
+  protected MouseInputListener mouseInputListener;
+
+  /** The key listener listening to the list */
+  private KeyHandler keyListener;
 
   /** The property change listener listening to the list. */
-  protected PropertyChangeHandler propertyChangeListener;
+  protected PropertyChangeListener propertyChangeListener;
+
 
   /** The component listener that receives notification for resizing the
    * JList component.*/
@@ -498,7 +582,8 @@ public class BasicListUI extends ListUI
   void damageLayout()
   {
     updateLayoutStateNeeded = 1;
-    list.revalidate();
+    if (list != null)
+      list.revalidate();
   }
 
   /**
@@ -523,6 +608,7 @@ public class BasicListUI extends ListUI
     listDataListener = new ListDataHandler();
     listSelectionListener = new ListSelectionHandler();
     mouseInputListener = new MouseInputHandler();
+    keyListener = new KeyHandler();
     propertyChangeListener = new PropertyChangeHandler();
     componentListener = new ComponentHandler();
     updateLayoutStateNeeded = 1;
@@ -570,6 +656,7 @@ public class BasicListUI extends ListUI
     list.getModel().addListDataListener(listDataListener);
     list.addListSelectionListener(listSelectionListener);
     list.addMouseListener(mouseInputListener);
+    list.addKeyListener(keyListener);
     list.addMouseMotionListener(mouseInputListener);
     list.addPropertyChangeListener(propertyChangeListener);
     list.addComponentListener(componentListener);
@@ -584,6 +671,7 @@ public class BasicListUI extends ListUI
     list.getModel().removeListDataListener(listDataListener);
     list.removeListSelectionListener(listSelectionListener);
     list.removeMouseListener(mouseInputListener);
+    list.removeKeyListener(keyListener);
     list.removeMouseMotionListener(mouseInputListener);
     list.removePropertyChangeListener(propertyChangeListener);
   }
