@@ -56,6 +56,7 @@ import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.beans.PropertyVetoException;
 
+import javax.swing.BorderFactory;
 import javax.swing.Box;
 import javax.swing.BoxLayout;
 import javax.swing.Icon;
@@ -84,8 +85,12 @@ public class BasicOptionPaneUI extends OptionPaneUI
   /**
    * This is a helper class that listens to the buttons located at the bottom
    * of the JOptionPane.
+   *
+   * @specnote Apparently this class was intended to be protected,
+   *           but was made public by a compiler bug and is now
+   *           public for compatibility.
    */
-  protected class ButtonActionListener implements ActionListener
+  public class ButtonActionListener implements ActionListener
   {
     /** The index of the option this button represents. */
     protected int buttonIndex;
@@ -151,8 +156,13 @@ public class BasicOptionPaneUI extends OptionPaneUI
    * This helper layout manager is responsible for the layout of the button
    * area. The button area is the panel that holds the buttons which
    * represent the options.
+   *
+   * @specnote Apparently this class was intended to be protected,
+   *           but was made public by a compiler bug and is now
+   *           public for compatibility.
    */
-  protected class ButtonAreaLayout implements LayoutManager
+  // FIXME: should be static
+  public class ButtonAreaLayout implements LayoutManager
   {
     /** Whether this layout will center the buttons. */
     protected boolean centersChildren = true;
@@ -365,6 +375,10 @@ public class BasicOptionPaneUI extends OptionPaneUI
 
   /**
    * This helper class handles property change events from the JOptionPane.
+   *
+   * @specnote Apparently this class was intended to be protected,
+   *           but was made public by a compiler bug and is now
+   *           public for compatibility.
    */
   public class PropertyChangeHandler implements PropertyChangeListener
   {
@@ -452,19 +466,15 @@ public class BasicOptionPaneUI extends OptionPaneUI
   private transient Border buttonBorder;
 
   /** The string used to describe OK buttons. */
-  // FIXME: wrong name for a constant.
   private static final String OK_STRING = "OK";
 
   /** The string used to describe Yes buttons. */
-  // FIXME: wrong name for a constant.
   private static final String YES_STRING = "Yes";
 
   /** The string used to describe No buttons. */
-  // FIXME: wrong name for a constant.
   private static final String NO_STRING = "No";
 
   /** The string used to describe Cancel buttons. */
-  // FIXME: wrong name for a constant.
   private static final String CANCEL_STRING = "Cancel";
 
   /** The container for the message area.
@@ -854,6 +864,7 @@ public class BasicOptionPaneUI extends OptionPaneUI
     addIcon(messageArea);
 
     JPanel rightSide = new JPanel();
+    rightSide.setBorder(BorderFactory.createEmptyBorder(0, 11, 17, 0));
     rightSide.setLayout(new GridBagLayout());
     GridBagConstraints con = createConstraints();
 
@@ -864,19 +875,16 @@ public class BasicOptionPaneUI extends OptionPaneUI
       {
 	Object[] selection = optionPane.getSelectionValues();
 
-//	if (selection == null)
-//	  inputComponent = new JTextField();
-//	else if (selection.length < 20)
-//	  inputComponent = new JComboBox(selection);
-	// FIXME: Uncomment when the widgets are done.
 	if (selection == null)
-	  inputComponent = null;
+          inputComponent = new JTextField(15);
+	else if (selection.length < 20)
+          inputComponent = new JComboBox(selection);
 	else
 	  inputComponent = new JList(selection);
 	if (inputComponent != null)
 	  {
 	    addMessageComponents(rightSide, con, inputComponent,
-	                         getMaxCharactersPerLineCount(), true);
+                                 getMaxCharactersPerLineCount(), false);
 	    resetSelectedValue();
 	    selectInitialValue(optionPane);
 	  }
@@ -906,7 +914,7 @@ public class BasicOptionPaneUI extends OptionPaneUI
    */
   protected Container createSeparator()
   {
-    return null;
+    return (Container) Box.createVerticalStrut(17);
   }
 
   /**
@@ -1123,6 +1131,7 @@ public class BasicOptionPaneUI extends OptionPaneUI
 	optionPane.add(button);
       }
 
+    optionPane.setBorder(BorderFactory.createEmptyBorder(12, 12, 11, 11));
     optionPane.invalidate();
   }
 
