@@ -1,5 +1,5 @@
 /* BasicProgressBarUI.java --
-   Copyright (C) 2004 Free Software Foundation, Inc.
+   Copyright (C) 2004, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -48,6 +48,9 @@ import java.awt.Point;
 import java.awt.Rectangle;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
+import java.awt.font.FontRenderContext;
+import java.awt.geom.AffineTransform;
+import java.awt.geom.Rectangle2D;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 
@@ -72,8 +75,12 @@ public class BasicProgressBarUI extends ProgressBarUI
   /**
    * A helper class that listens for ChangeEvents 
    * from the progressBar's model.
+   *
+   * @specnote Apparently this class was intended to be protected,
+   *           but was made public by a compiler bug and is now
+   *           public for compatibility.
    */
-  protected class ChangeHandler implements ChangeListener
+  public class ChangeHandler implements ChangeListener
   {
     /**
      * Called every time the state of the model changes.
@@ -103,7 +110,7 @@ public class BasicProgressBarUI extends ProgressBarUI
     {
       // Only need to listen for indeterminate changes.
       // All other things are done on a repaint.
-      if (e.getPropertyName().equals(JProgressBar.INDETERMINATE_CHANGED_PROPERTY))
+      if (e.getPropertyName().equals("inderterminate"))
 	if (((Boolean) e.getNewValue()).booleanValue())
 	  startAnimationTimer();
 	else
@@ -381,16 +388,16 @@ public class BasicProgressBarUI extends ProgressBarUI
   {
     // The only thing we need to worry about is
     // the text size.
-    Graphics g = progressBar.getGraphics();
-
     Insets insets = c.getInsets();
 
-    FontMetrics fm = g.getFontMetrics(c.getFont());
-
-    int textW = fm.stringWidth(progressBar.getString());
-    int textH = fm.getHeight();
-
-    g.dispose();
+    // make a fontrenderer context so that we can make assumptions about
+    // the string bounds
+    FontRenderContext ctx = new FontRenderContext(new AffineTransform(),
+                                                  false, false);
+    Rectangle2D bounds = c.getFont().getStringBounds(progressBar.getString(),
+                                                     ctx);
+    int textW = (int) bounds.getWidth();
+    int textH = (int) bounds.getHeight();
 
     if (progressBar.getOrientation() == JProgressBar.HORIZONTAL)
       {
@@ -467,7 +474,7 @@ public class BasicProgressBarUI extends ProgressBarUI
   /**
    * This method increments the animation index.
    */
-  public void incrementAnimationIndex()
+  protected void incrementAnimationIndex()
   {
     animationIndex++;
     //numFrames is like string length, it should be named numFrames or something
@@ -489,11 +496,6 @@ public class BasicProgressBarUI extends ProgressBarUI
       paintDeterminate(g, c);
     else
       paintIndeterminate(g, c);
-      
-    if (progressBar.isBorderPainted())
-      progressBar.getBorder().paintBorder(progressBar, g, 0, 0,
-                                          progressBar.getWidth(),
-					  progressBar.getHeight());
   }
 
   /**
