@@ -1,5 +1,5 @@
-/* BasicSplitPaneUI.java
-   Copyright (C) 2003, 2004 Free Software Foundation, Inc.
+/* BasicSplitPaneUI.java --
+   Copyright (C) 2003, 2004, 2005 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -35,6 +35,7 @@ this exception to your version of the library, but you are not
 obligated to do so.  If you do not wish to do so, delete this
 exception statement from your version. */
 
+
 package javax.swing.plaf.basic;
 
 import java.awt.Canvas;
@@ -51,10 +52,9 @@ import java.awt.event.ActionListener;
 import java.awt.event.FocusAdapter;
 import java.awt.event.FocusEvent;
 import java.awt.event.FocusListener;
-import java.awt.event.MouseEvent;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
-import javax.swing.JButton;
+
 import javax.swing.JComponent;
 import javax.swing.JSplitPane;
 import javax.swing.KeyStroke;
@@ -63,7 +63,6 @@ import javax.swing.UIManager;
 import javax.swing.plaf.ComponentUI;
 import javax.swing.plaf.SplitPaneUI;
 
-
 /**
  * This is the Basic Look and Feel implementation of the SplitPaneUI  class.
  */
@@ -72,8 +71,12 @@ public class BasicSplitPaneUI extends SplitPaneUI
   /**
    * This Layout Manager controls the position and size of the components when
    * the JSplitPane's orientation is HORIZONTAL_SPLIT.
+   *
+   * @specnote Apparently this class was intended to be protected,
+   *           but was made public by a compiler bug and is now
+   *           public for compatibility.
    */
-  protected class BasicHorizontalLayoutManager implements LayoutManager2
+  public class BasicHorizontalLayoutManager implements LayoutManager2
   {
     // 3 components at a time.
     // LEFT/TOP = 0
@@ -345,7 +348,7 @@ public class BasicSplitPaneUI extends SplitPaneUI
 		    height = Math.max(height, dims.height);
 	        }
 	    }
-	  return new Dimension(500, 500); //width, height);	
+	  return new Dimension(width, height);	
         }
       return null;
     }
@@ -486,8 +489,12 @@ public class BasicSplitPaneUI extends SplitPaneUI
   /**
    * This class is the Layout Manager for the JSplitPane when the orientation
    * is VERTICAL_SPLIT.
+   *
+   * @specnote Apparently this class was intended to be protected,
+   *           but was made public by a compiler bug and is now
+   *           public for compatibility.
    */
-  protected class BasicVerticalLayoutManager
+  public class BasicVerticalLayoutManager
     extends BasicHorizontalLayoutManager
   {
     /**
@@ -606,7 +613,7 @@ public class BasicSplitPaneUI extends SplitPaneUI
 		  width = Math.max(width, dims.width);
 	        }
 	    }
-	  return new Dimension(500, 500); //width, height);
+	  return new Dimension(width, height);
         }
       return null;
     }
@@ -654,8 +661,12 @@ public class BasicSplitPaneUI extends SplitPaneUI
 
   /**
    * This class handles FocusEvents from the JComponent.
+   *
+   * @specnote Apparently this class was intended to be protected,
+   *           but was made public by a compiler bug and is now
+   *           public for compatibility.
    */
-  protected class FocusHandler extends FocusAdapter
+  public class FocusHandler extends FocusAdapter
   {
     /**
      * This method is called when the JSplitPane gains focus.
@@ -681,8 +692,12 @@ public class BasicSplitPaneUI extends SplitPaneUI
   /**
    * This is a deprecated class. It is supposed to be used for handling down
    * and right key presses.
+   *
+   * @specnote Apparently this class was intended to be protected,
+   *           but was made public by a compiler bug and is now
+   *           public for compatibility.
    */
-  protected class KeyboardDownRightHandler implements ActionListener
+  public class KeyboardDownRightHandler implements ActionListener
   {
     /**
      * This method is called when the down or right keys are pressed.
@@ -698,8 +713,12 @@ public class BasicSplitPaneUI extends SplitPaneUI
   /**
    * This is a deprecated class. It is supposed to be used for handling end
    * key presses.
+   *
+   * @specnote Apparently this class was intended to be protected,
+   *           but was made public by a compiler bug and is now
+   *           public for compatibility.
    */
-  protected class KeyboardEndHandler implements ActionListener
+  public class KeyboardEndHandler implements ActionListener
   {
     /**
      * This method is called when the end key is pressed.
@@ -715,8 +734,12 @@ public class BasicSplitPaneUI extends SplitPaneUI
   /**
    * This is a deprecated class. It is supposed to be used for handling home
    * key presses.
+   *
+   * @specnote Apparently this class was intended to be protected,
+   *           but was made public by a compiler bug and is now
+   *           public for compatibility.
    */
-  protected class KeyboardHomeHandler implements ActionListener
+  public class KeyboardHomeHandler implements ActionListener
   {
     /**
      * This method is called when the home key is pressed.
@@ -732,8 +755,12 @@ public class BasicSplitPaneUI extends SplitPaneUI
   /**
    * This is a deprecated class. It is supposed to be used for handling resize
    * toggles.
+   *
+   * @specnote Apparently this class was intended to be protected,
+   *           but was made public by a compiler bug and is now
+   *           public for compatibility.
    */
-  protected class KeyboardResizeToggleHandler implements ActionListener
+  public class KeyboardResizeToggleHandler implements ActionListener
   {
     /**
      * This method is called when a resize is toggled.
@@ -749,8 +776,12 @@ public class BasicSplitPaneUI extends SplitPaneUI
   /**
    * This is a deprecated class. It is supposed to be used for handler up and
    * left key presses.
+   *
+   * @specnote Apparently this class was intended to be protected,
+   *           but was made public by a compiler bug and is now
+   *           public for compatibility.
    */
-  protected class KeyboardUpLeftHandler implements ActionListener
+  public class KeyboardUpLeftHandler implements ActionListener
   {
     /**
      * This method is called when the left or up keys are pressed.
@@ -766,8 +797,12 @@ public class BasicSplitPaneUI extends SplitPaneUI
   /**
    * This helper class handles PropertyChangeEvents from the JSplitPane. When
    * a property changes, this will update the UI accordingly.
+   *
+   * @specnote Apparently this class was intended to be protected,
+   *           but was made public by a compiler bug and is now
+   *           public for compatibility.
    */
-  protected class PropertyHandler implements PropertyChangeListener
+  public class PropertyHandler implements PropertyChangeListener
   {
     /**
      * This method is called whenever one of the JSplitPane's properties
@@ -846,43 +881,43 @@ public class BasicSplitPaneUI extends SplitPaneUI
   /** The JSplitPane's focus handler. */
   protected FocusListener focusListener;
 
-  /** Deprecated. The handler for down and right key presses. */
+  /** @deprecated The handler for down and right key presses. */
   protected ActionListener keyboardDownRightListener;
 
-  /** Deprecated. The handler for end key presses. */
+  /** @deprecated The handler for end key presses. */
   protected ActionListener keyboardEndListener;
 
-  /** Deprecated. The handler for home key presses. */
+  /** @deprecated The handler for home key presses. */
   protected ActionListener keyboardHomeListener;
 
-  /** Deprecated. The handler for toggling resizes. */
+  /** @deprecated The handler for toggling resizes. */
   protected ActionListener keyboardResizeToggleListener;
 
-  /** Deprecated. The handler for up and left key presses. */
+  /** @deprecated The handler for up and left key presses. */
   protected ActionListener keyboardUpLeftListener;
 
   /** The JSplitPane's current layout manager. */
   protected BasicHorizontalLayoutManager layoutManager;
 
-  /** Deprecated. The divider resize toggle key. */
+  /** @deprecated The divider resize toggle key. */
   protected KeyStroke dividerResizeToggleKey;
 
-  /** Deprecated. The down key. */
+  /** @deprecated The down key. */
   protected KeyStroke downKey;
 
-  /** Deprecated. The end key. */
+  /** @deprecated The end key. */
   protected KeyStroke endKey;
 
-  /** Deprecated. The home key. */
+  /** @deprecated The home key. */
   protected KeyStroke homeKey;
 
-  /** Deprecated. The left key. */
+  /** @deprecated The left key. */
   protected KeyStroke leftKey;
 
-  /** Deprecated. The right key. */
+  /** @deprecated The right key. */
   protected KeyStroke rightKey;
 
-  /** Deprecated. The up key. */
+  /** @deprecated The up key. */
   protected KeyStroke upKey;
 
   /** Set to true when dragging heavy weight components. */
@@ -955,8 +990,8 @@ public class BasicSplitPaneUI extends SplitPaneUI
    */
   protected void installDefaults()
   {
-    resetLayoutManager();
     divider = createDefaultDivider();
+    resetLayoutManager();
     nonContinuousLayoutDivider = createDefaultNonContinuousLayoutDivider();
     splitPane.add(divider, JSplitPane.DIVIDER);
 
@@ -1043,10 +1078,11 @@ public class BasicSplitPaneUI extends SplitPaneUI
   }
 
   /**
-   * Deprecated. This method creates a new ActionListener for up and left key
-   * presses.
+   * This method creates a new ActionListener for up and left key presses.
    *
    * @return A new ActionListener for up and left keys.
+   *
+   * @deprecated 1.3
    */
   protected ActionListener createKeyboardUpLeftListener()
   {
@@ -1054,10 +1090,11 @@ public class BasicSplitPaneUI extends SplitPaneUI
   }
 
   /**
-   * Deprecated. This method creates a new ActionListener for down and right
-   * key presses.
+   * This method creates a new ActionListener for down and right key presses.
    *
    * @return A new ActionListener for down and right keys.
+   *
+   * @deprecated 1.3
    */
   protected ActionListener createKeyboardDownRightListener()
   {
@@ -1065,10 +1102,11 @@ public class BasicSplitPaneUI extends SplitPaneUI
   }
 
   /**
-   * Deprecated. This method creates a new ActionListener for home key
-   * presses.
+   * This method creates a new ActionListener for home key presses.
    *
    * @return A new ActionListener for home keys.
+   *
+   * @deprecated
    */
   protected ActionListener createKeyboardHomeListener()
   {
@@ -1076,9 +1114,11 @@ public class BasicSplitPaneUI extends SplitPaneUI
   }
 
   /**
-   * Deprecated. This method creates a new ActionListener for end key presses.
+   * This method creates a new ActionListener for end key presses.i
    *
    * @return A new ActionListener for end keys.
+   *
+   * @deprecated 1.3
    */
   protected ActionListener createKeyboardEndListener()
   {
@@ -1086,10 +1126,11 @@ public class BasicSplitPaneUI extends SplitPaneUI
   }
 
   /**
-   * Depcreated. This method creates a new ActionListener for resize toggle
-   * key events.
+   * This method creates a new ActionListener for resize toggle key events.
    *
    * @return A new ActionListener for resize toggle keys.
+   *
+   * @deprecated 1.3
    */
   protected ActionListener createKeyboardResizeToggleListener()
   {
@@ -1343,7 +1384,6 @@ public class BasicSplitPaneUI extends SplitPaneUI
    */
   public void paint(Graphics g, JComponent jc)
   {
-    // Do nothing. All the painting is handled by children.
   }
 
   /**
@@ -1404,9 +1444,8 @@ public class BasicSplitPaneUI extends SplitPaneUI
       layoutManager = new BasicHorizontalLayoutManager();
     else
       layoutManager = new BasicVerticalLayoutManager();
-    layoutManager.invalidateLayout(splitPane);
-    layoutManager.updateComponents();
     getSplitPane().setLayout(layoutManager);
+    layoutManager.updateComponents();
 
     // invalidating by itself does not invalidate the layout.
     getSplitPane().revalidate();
@@ -1483,10 +1522,11 @@ public class BasicSplitPaneUI extends SplitPaneUI
   }
 
   /**
-   * Deprecated. This method returns the width of one of the sides of the
-   * divider's border.
+   * This method returns the width of one of the sides of the divider's border.
    *
    * @return The width of one side of the divider's border.
+   *
+   * @deprecated 1.3
    */
   protected int getDividerBorderSize()
   {
