@@ -1,5 +1,5 @@
 /* DefaultTableCellRenderer.java --
-   Copyright (C) 2002, 2004 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -122,10 +122,14 @@ public class DefaultTableCellRenderer extends JLabel
                                                  boolean hasFocus,
                                                  int row, int column)
   {
-    if (value!=null)
+    if (value != null)
       super.setText(value.toString());
 
     setOpaque(true);
+
+    if (table == null)
+      return this;
+
     if (isSelected)
       {
         setBackground(table.getSelectionBackground());
@@ -199,8 +203,8 @@ public class DefaultTableCellRenderer extends JLabel
    * <p>This method needs to be overridden in a subclass to actually
    * do something.</p>
    */
-  public void firePropertyChange(String propertyName, Object oldValue,
-                                 Object newValue)
+  protected void firePropertyChange(String propertyName, Object oldValue,
+                                    Object newValue)
   {
     // Does nothing.
   }
