@@ -50,6 +50,7 @@ import java.awt.datatransfer.StringSelection;
 import java.awt.datatransfer.Transferable;
 import java.awt.datatransfer.UnsupportedFlavorException;
 import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
 import java.awt.event.InputMethodListener;
 import java.awt.event.KeyEvent;
 import java.io.IOException;
@@ -68,6 +69,8 @@ import javax.swing.JComponent;
 import javax.swing.JViewport;
 import javax.swing.KeyStroke;
 import javax.swing.Scrollable;
+import javax.swing.SwingConstants;
+import javax.swing.Timer;
 import javax.swing.TransferHandler;
 import javax.swing.UIManager;
 import javax.swing.event.CaretEvent;
@@ -296,6 +299,44 @@ public abstract class JTextComponent extends JComponent
     }
   }
 
+  /**
+   * The timer that lets the caret blink.
+   */
+  private class CaretBlinkTimer
+    extends Timer
+    implements ActionListener
+  {
+    /**
+     * Creates a new CaretBlinkTimer object with a default delay of 1 second.
+     */
+    public CaretBlinkTimer()
+    {
+      super(1000, null);
+      addActionListener(this);
+    }
+
+    /**
+     * Lets the caret blink.
+     */
+    public void actionPerformed(ActionEvent ev)
+    {
+      caret.setVisible(!caret.isVisible());
+    }
+
+    /**
+     * Updates the blink delay according to the current caret.
+     */
+    public void update()
+    {
+      stop();
+      setDelay(caret.getBlinkRate());
+      if (editable)
+        start();
+      else
+        caret.setVisible(false);
+    }
+  }
+
   /**
    * According to <a
    * href="http://java.sun.com/products/jfc/tsc/special_report/kestrel/keybindings.html">this
@@ -652,7 +693,9 @@ public abstract class JTextComponent extends JComponent
   private Keymap keymap;
   private char focusAccelerator = '\0';
   private NavigationFilter navigationFilter;
-  
+
+  private CaretBlinkTimer caretBlinkTimer;
+
   /**
    * Get a Keymap from the global keymap table, by name.
    *
@@ -895,30 +938,11 @@ public abstract class JTextComponent extends JComponent
   private Insets margin;
   private boolean dragEnabled;
 
-  /** Issues repaint request on document changes. */
-  private DocumentListener repaintListener;
-
   /**
    * Creates a new <code>JTextComponent</code> instance.
    */
   public JTextComponent()
   {
-    repaintListener = new DocumentListener()
-      {
-	public void changedUpdate(DocumentEvent ev)
-	{
-	  repaint();
-	}
-	public void insertUpdate(DocumentEvent ev)
-	{
-	  repaint();
-	}
-	public void removeUpdate(DocumentEvent ev)
-	{
-	  repaint();
-	}
-      };
-
     Keymap defkeymap = getKeymap(DEFAULT_KEYMAP);
     boolean creatingKeymap = false;
     if (defkeymap == null)
@@ -928,7 +952,10 @@ public abstract class JTextComponent extends JComponent
         creatingKeymap = true;
       }
 
+    caretBlinkTimer = new CaretBlinkTimer();
+
     setFocusable(true);
+    setEditable(true);
     enableEvents(AWTEvent.KEY_EVENT_MASK);
     updateUI();
     
@@ -952,13 +979,6 @@ public abstract class JTextComponent extends JComponent
   {
     Document oldDoc = doc;
     doc = newDoc;
-
-    // setup document listener
-    if (oldDoc != null)
-      oldDoc.removeDocumentListener(repaintListener);
-    if (newDoc != null)
-      newDoc.addDocumentListener(repaintListener);
-
     firePropertyChange("document", oldDoc, newDoc);
     revalidate();
     repaint();
@@ -1111,19 +1131,39 @@ public abstract class JTextComponent extends JComponent
 
   public Dimension getPreferredScrollableViewportSize()
   {
-    return null;
+    return getPreferredSize();
   }
 
   public int getScrollableUnitIncrement(Rectangle visible, int orientation,
                                         int direction)
   {
-    return 0;
+    // We return 1/10 of the visible area as documented in Sun's API docs.
+    if (orientation == SwingConstants.HORIZONTAL)
+      return visible.width / 10;
+    else if (orientation == SwingConstants.VERTICAL)
+      return visible.height / 10;
+    else
+      throw new IllegalArgumentException("orientation must be either "
+                                      + "javax.swing.SwingConstants.VERTICAL "
+                                      + "or "
+                                      + "javax.swing.SwingConstants.HORIZONTAL"
+                                         );
   }
 
   public int getScrollableBlockIncrement(Rectangle visible, int orientation,
                                          int direction)
   {
-    return 0;
+    // We return the whole visible area as documented in Sun's API docs.
+    if (orientation == SwingConstants.HORIZONTAL)
+      return visible.width;
+    else if (orientation == SwingConstants.VERTICAL)
+      return visible.height;
+    else
+      throw new IllegalArgumentException("orientation must be either "
+                                      + "javax.swing.SwingConstants.VERTICAL "
+                                      + "or "
+                                      + "javax.swing.SwingConstants.HORIZONTAL"
+                                         );
   }
 
   /**
@@ -1146,6 +1186,14 @@ public abstract class JTextComponent extends JComponent
     if (editable == newValue)
       return;
 
+    if (newValue == true)
+      caretBlinkTimer.start();
+    else
+      {
+        caretBlinkTimer.stop();
+        caret.setVisible(false);
+      }
+
     boolean oldValue = editable;
     editable = newValue;
     firePropertyChange("editable", oldValue, newValue);
@@ -1174,6 +1222,8 @@ public abstract class JTextComponent extends JComponent
     Caret oldCaret = caret;
     caret = newCaret;
 
+    caretBlinkTimer.update();
+
     if (caret != null)
       caret.install(this);
     
