@@ -1,5 +1,5 @@
 /* PlainView.java -- 
-   Copyright (C) 2004  Free Software Foundation, Inc.
+   Copyright (C) 2004, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -46,7 +46,6 @@ import java.awt.Graphics;
 import java.awt.Rectangle;
 import java.awt.Shape;
 
-
 public class PlainView extends View
   implements TabExpander
 {
@@ -93,6 +92,9 @@ public class PlainView extends View
   public Shape modelToView(int position, Shape a, Position.Bias b)
     throws BadLocationException
   {
+    // Ensure metrics are up-to-date.
+    updateMetrics();
+    
     Document document = getDocument();
 
     // Get rectangle of the line containing position.
@@ -115,13 +117,14 @@ public class PlainView extends View
     return rect;
   }
   
-  public void drawLine(int lineIndex, Graphics g, int x, int y)
+  protected void drawLine(int lineIndex, Graphics g, int x, int y)
   {
     try
       {
 	metrics = g.getFontMetrics();
 	// FIXME: Selected text are not drawn yet.
-	drawUnselectedText(g, x, y, 0, getDocument().getLength());
+	Element line = getElement().getElement(lineIndex);
+	drawUnselectedText(g, x, y, line.getStartOffset(), line.getEndOffset());
 	//drawSelectedText(g, , , , );
       }
     catch (BadLocationException e)
@@ -145,11 +148,14 @@ public class PlainView extends View
     g.setColor(unselectedColor);
     Segment segment = new Segment();
     getDocument().getText(p0, p1 - p0, segment);
-    return Utilities.drawTabbedText(segment, x, y, g, this, 0);
+    return Utilities.drawTabbedText(segment, x, y, g, this, segment.offset);
   }
 
   public void paint(Graphics g, Shape s)
   {
+    // Ensure metrics are up-to-date.
+    updateMetrics();
+    
     JTextComponent textComponent = (JTextComponent) getContainer();
 
     g.setFont(textComponent.getFont());
@@ -159,10 +165,18 @@ public class PlainView extends View
     Rectangle rect = s.getBounds();
 
     // FIXME: Text may be scrolled.
-    drawLine(0, g, rect.x, rect.y);
+    Document document = textComponent.getDocument();
+    Element root = document.getDefaultRootElement();
+    int y = rect.y;
+    
+    for (int i = 0; i < root.getElementCount(); i++)
+      {
+	drawLine(i, g, rect.x, y);
+	y += metrics.getHeight();
+      }
   }
 
-  public int getTabSize()
+  protected int getTabSize()
   {
     return 8;
   }
@@ -186,7 +200,42 @@ public class PlainView extends View
     if (axis != X_AXIS && axis != Y_AXIS)
       throw new IllegalArgumentException();
 
-    return 10;
+    // make sure we have the metrics
+    updateMetrics();
+
+    float span = 0;
+    Element el = getElement();
+    Document doc = el.getDocument();
+    Segment seg = new Segment();
+
+    switch (axis)
+      {
+      case X_AXIS:
+        // calculate the maximum of the line's widths
+        for (int i = 0; i < el.getElementCount(); i++)
+          {
+            Element child = el.getElement(i);
+            int start = child.getStartOffset();
+            int end = child.getEndOffset();
+            try {
+              doc.getText(start, start + end, seg);
+            }
+            catch (BadLocationException ex)
+              {
+                // throw new ClasspathAssertionError
+                // ("no BadLocationException should be thrown here");
+              }
+            int width = metrics.charsWidth(seg.array, seg.offset, seg.count);
+            span = Math.max(span, width);
+          }
+        break;
+      case Y_AXIS:
+      default:
+        span = metrics.getHeight() * el.getElementCount();
+        break;
+      }
+
+    return span;
   }
 }
 
