@@ -35,6 +35,7 @@ this exception to your version of the library, but you are not
 obligated to do so.  If you do not wish to do so, delete this
 exception statement from your version. */
 
+
 package javax.swing.text;
 
 import java.awt.Color;
@@ -45,13 +46,14 @@ import java.io.IOException;
 import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
 import java.io.Serializable;
-import javax.swing.event.EventListenerList;
-import javax.swing.event.ChangeEvent;
-import javax.swing.event.ChangeListener;
 import java.util.Enumeration;
 import java.util.EventListener;
 import java.util.Hashtable;
 
+import javax.swing.event.ChangeEvent;
+import javax.swing.event.ChangeListener;
+import javax.swing.event.EventListenerList;
+
 public class StyleContext 
     implements Serializable, AbstractDocument.AttributeContext
 {
@@ -201,7 +203,11 @@ public class StyleContext
 
     public void setResolveParent(AttributeSet parent)
     {
-      attributes = StyleContext.this.addAttribute(attributes, ResolveAttribute, parent);
+      if (parent != null)
+        {
+          attributes = StyleContext.this.addAttribute
+            (attributes, ResolveAttribute, parent);
+        }
       fireStateChanged();
     }
       
