@@ -66,30 +66,6 @@ final class VMClassLoader
   /**
    * Helper to define a class using a string of bytes. This assumes that
    * the security checks have already been performed, if necessary.
-   * <strong>This method will be removed in a future version of GNU
-   * Classpath</strong>.
-   *
-   * @param name the name to give the class, or null if unknown
-   * @param data the data representing the classfile, in classfile format
-   * @param offset the offset into the data where the classfile starts
-   * @param len the length of the classfile data in the array
-   * @return the class that was defined
-   * @throws ClassFormatError if data is not in proper classfile format
-   * @deprecated Implement
-   * {@link #defineClass(ClassLoader, String, byte[], int, int, ProtectionDomain)}
-   *   instead.
-   */
-  static final native Class defineClass(ClassLoader cl, String name,
-                                        byte[] data, int offset, int len)
-    throws ClassFormatError;
-
-  /**
-   * Helper to define a class using a string of bytes. This assumes that
-   * the security checks have already been performed, if necessary.
-   *
-   * <strong>For backward compatibility, this just ignores the protection
-   * domain; that is the wrong behavior, and you should directly implement
-   * this method natively if you can.</strong>
    *
    * Implementations of this method are advised to consider the
    * situation where user code modifies the byte array after it has
@@ -105,13 +81,10 @@ final class VMClassLoader
    * @return the class that was defined
    * @throws ClassFormatError if data is not in proper classfile format
    */
-  static final Class defineClass(ClassLoader cl, String name,
+  static final native Class defineClass(ClassLoader cl, String name,
                                  byte[] data, int offset, int len,
                                  ProtectionDomain pd)
-    throws ClassFormatError
-  {
-    return defineClass(cl, name, data, offset, len);
-  }
+    throws ClassFormatError;
 
   /**
    * Helper to resolve all references to other classes from this class.
@@ -123,19 +96,14 @@ final class VMClassLoader
   /**
    * Helper to load a class from the bootstrap class loader.
    *
-   * XXX - Not implemented; this requires native help.
-   *
    * @param name the class name to load
    * @param resolve whether to resolve it
    * @return the class, loaded by the bootstrap classloader or null
    * if the class wasn't found. Returning null is equivalent to throwing
    * a ClassNotFoundException (but a possible performance optimization).
    */
-  static final Class loadClass(String name, boolean resolve)
-    throws ClassNotFoundException
-  {
-    return null;
-  }
+  static final native Class loadClass(String name, boolean resolve)
+    throws ClassNotFoundException;
 
   /**
    * Helper to load a resource from the bootstrap class loader.
@@ -262,66 +230,10 @@ final class VMClassLoader
    * <li>'V' - void</li>
    * </ul>
    *
-   * Note that this is currently a java version that converts the type code
-   * to a string and calls the native <code>getPrimitiveClass(String)</code>
-   * method for backwards compatibility with VMs that used old versions of
-   * GNU Classpath. Please replace this method with a native method
-   * <code>final static native Class getPrimitiveClass(char type);</code>
-   * if your VM supports it. <strong>The java version of this method and
-   * the String version of this method will disappear in a future version
-   * of GNU Classpath</strong>.
-   *
    * @param type the primitive type
    * @return a "bogus" class representing the primitive type
    */
-  static final Class getPrimitiveClass(char type)
-  {
-    String t;
-    switch (type)
-      {
-      case 'Z':
-        t = "boolean";
-        break;
-      case 'B':
-        t = "byte";
-        break;
-      case 'C':
-        t = "char";
-        break;
-      case 'D':
-        t = "double";
-        break;
-      case 'F':
-        t = "float";
-        break;
-      case 'I':
-        t = "int";
-        break;
-      case 'J':
-        t = "long";
-        break;
-      case 'S':
-        t = "short";
-        break;
-      case 'V':
-        t = "void";
-        break;
-      default:
-        throw new NoClassDefFoundError("Invalid type specifier: " + type);
-      }
-    return getPrimitiveClass(t);
-  }
-
-  /**
-   * Old version of the interface, added here for backwards compatibility.
-   * Called by the java version of getPrimitiveClass(char) when no native
-   * version of that method is available.
-   * <strong>This method will be removed in a future version of GNU
-   * Classpath</strong>.
-   * @param type the primitive type
-   * @return a "bogus" class representing the primitive type
-   */
-  static final native Class getPrimitiveClass(String type);
+  static final native Class getPrimitiveClass(char type);
 
   /**
    * The system default for assertion status. This is used for all system
