@@ -37,9 +37,13 @@ exception statement from your version. */
 
 package java.lang;
 
-import java.util.Properties;
-
-import java.io.*;
+import java.io.BufferedInputStream;
+import java.io.BufferedOutputStream;
+import java.io.FileDescriptor;
+import java.io.FileInputStream;
+import java.io.FileOutputStream;
+import java.io.InputStream;
+import java.io.PrintStream;
 
 /**
  * VMSystem is a package-private helper class for System that the
